/* 
**  Project       : Simple Local News Reader 
**
**	Name          : GETNEWS
**  Version       : 1.9
**
**	Author        : P.Goujard
**	Started       : 22/01/93
**  Last modified : 04 Nov 1993
**	Goal          : Getting unread Mail and News articles from the spool
**			            directory and putting them in a SOUP file
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>

#define UNSET 0
#define SET   1
#define ASK   2
#define OK    1

/* Values returned by lfgets() */
#define LFGETS_OK    0
#define LFGETS_EOF   1
#define LFGETS_ERR   2

#ifdef INFOCOM
#include "new.h"
#endif

/*
** Change those values to the one in your system
*/

#define BIN_ZIP "zip"
#define BIN_RM  "rm"

/*
** Usually on unix a user has a file called ".newsrc" in his home
** directory. 
** This file is created by newsreaders like, rn,nn,vn,tin,tass,trn
** and contains the list of newsgroups and last read pointers
*/
#define NEWSRC ".newsrc"

/*
** nrc2 is the name of the temporary .newsrc 
** You shouldn't need to change it
*/
#define NRC2   "nrc2"

/*
** LOGDIR is the directory where error and log files are created
** Usually /tmp or /usr/tmp
*/
#ifdef INFOCOM
#define LOGDIR "/server/logfiles"
#else
#define LOGDIR "/usr/tmp"
#endif
#define LOGFILE "getnews.log"

/*
** Under MMDF, a mailbox consists of messages separated by 4 Ctrl A
** Normal mailboxes just use the "From " line as separator
*/
#define MMDF_SEPARATOR "\001\001\001\001"

/*
** SPOOL is the directory where the news are located
** This has no effect with NNTP
** GROUPS is the file containing the description of each newsgroup
*/
#define SPOOL "/usr/spool/news"
#define GROUPS "/usr/lib/news/newsgroups"

/*
** The following names are consistent with the 
** SOUP packet format.
** You shouldn't need to change them.
** For more info report to the file soup12.doc
*/
#define NEWSTXT ".MSG"
#define AREALIST "AREAS"

/*
** PACKETNAME is used to define the first part of the name of the packet
** the extension is taken fron the compression used
*/
#define PACKETNAME "INFONEWS"


/* Prototypes */

#ifdef __STDC__
# define P(s) s
#else
# define P(s) ()
# define remove unlink
#endif

/* getnews.c */
int main P((int argc , char *argv []));
void dot2slash P((char * ));
unsigned long add_file P((char *, char * , int ));
char *get_description P((char *, char * ));
int is_unread P((int, char *, int ));
char *upperit P((char * ));
int get_token P((char *, char **, char ** ));
void read_config P((void ));
int su_get P((char *line, char * ));
int sua_get P((char *line, char * ));
void index_message P((int, char *, unsigned long, int, int ));
int exp_env P((char * ));
int lfgets P((char ** , int * , FILE * ));

#undef P


/*
** Global variables
*/
int Msgmax,Grpmax,Mmdf,Usenet_only,
	Mail_only,Delete_mailbox,Get_mail,Mark_read,Do_index,Index_only;
int nntp,Verbose,Debug;
char Compress[80],Packet_name[80],Work_dir[80],Arctype[10],Encoding,Index_type;

int main (argc, argv)
int argc;
char *argv[];
{
	FILE *rc,*rc2,*log,*areas,*mbox,*fw;
	char home[80],newsrc[80],nrc2[80], templine[255], slog[80];
	char *nrcline=NULL, *list_read;
	char newsgroup[80],*pos,newsdir[80],ngrp2[80],message[80];
	char mlist_f[80],mailbox[80];
	char mailbox_type,area_name[80];
	int c, current_area=1, nowrite=0, error=0;
	int i,last,message_cnt;
	int total_msg=0,message_no;
	int biggest;
	int nrcsize = 256, retv;
	struct dirent *ficdir;
	DIR *rep;
	unsigned long offset=0L;
	extern char *optarg;
#ifdef USE_NNTP
    extern unsigned nntp_first, nntp_last;
#endif
#ifdef INFOCOM
	struct user User;

	strcpy (User.login,getenv("LOGNAME"));
	if (testuser(&User,USERBASE) >  0)
	{
		if ((show_priv_no(3,&User)==0) || show_flag_no(13,&User)) 
		{
			/* Not a gold user or has the no getnews flag */
			fprintf (stderr,"Sorry, you are not allowed to use getnews\n");
			return (-1);
		}
	}
#endif
	nntp = Verbose = Debug = 0;
	
	if ((pos=getenv("HOME"))==NULL)
	{
		fprintf (stderr,"$HOME not defined\n");
		return(-1);
	}
	strcpy (home,pos);

	read_config();

    while ( (c=getopt(argc,argv,"i:e:IvDnsmc:")) != -1 )
    {
			switch ( c )
			{
        case 'i':
					Index_type = *optarg;
					if( Index_type == 'n' )
						Do_index = UNSET;
					else
						Do_index = SET;
					break;
				
				case 'e':
					Encoding = *optarg;
					break;

				case 'I':
					Index_only = SET;
					break;

        case 'v':
					Verbose++;
					break;

        case 'D':
					Debug++;
					break;

        case 'n': /* use nntp for read news */
#ifdef USE_NNTP
					nntp++;
#else
					fprintf( stderr, "NNTP support unavailable.\n"
						"Recompile getnews.c with -DUSE_NNTP\n");
#endif
					break;

				case 's': /* don't save mail */
					Get_mail = UNSET;
					break;

				case 'm': /* don't mark news as read */
					Mark_read = UNSET;
					break;

				case 'c': /* define compress program */
					strcpy( Compress, optarg );
					if ( access( Compress, 1 )==-1 )
							Compress[0]='\0';
					break;
				
				case '?':
					error++;
					break;
			}
    }
		if( error )
		{
			fprintf( stderr, "usage: getnews [-iIvDnsmc]\n"
				"i ind_type : Use ind_type in indexes [cCin]\n"
				"e enc_type : Encode messages with enc_type format [umMbBn]\n"
				"I : Do ONLY index\n"
				"v : Verbose mode\n"
				"D : Debug mode\n"
				"n : NNTP mode\n"
				"s : Do not save mail\n"
				"m : Do not mark articles as read\n"
				"c compress_program : define compression program\n"
				"Command line parameters take precedence over the config file\n");
				return( -1 );
		}

#ifdef USE_NNTP
    if ( nntp )
        nntp_startup();
#endif

	if (Index_only==SET)
		Do_index=SET;

	if (Mmdf==SET)
		mailbox_type='M';
	else
		mailbox_type='m';

	if (Compress[0]=='\0') /* unset */
	{
		strcpy(Compress, BIN_ZIP );
		fprintf(stderr,"WARNING: undefined compression utility, using %s\n",
			Compress);
	}


	/*
	Opening of Files
	*/

	/* remove the AREAS file */
	sprintf (templine,"%s/%s",Work_dir,AREALIST);
	remove(templine);

	sprintf (newsrc,"%s/%s",home,NEWSRC);
	sprintf (nrc2,"%s/%s",Work_dir,NRC2);

	if((rc=fopen(newsrc,"r"))==NULL)
	{
		fprintf (stderr,"File [%s] not found\n",newsrc);
		return(1);
	}

	sprintf (slog,"%s/%s",LOGDIR,LOGFILE);
	log=fopen (slog,"a");
	if (log==NULL)
		log=stderr;

	sprintf (mlist_f,"%s/%s",Work_dir,AREALIST);
	if ((areas=fopen(mlist_f,"w"))==NULL)
	{
		fprintf (stderr,"Unable to create [%s] file \n",mlist_f);
		return(1);
	}


	/*
	** Create the name of the first area
	*/
	sprintf (area_name,"%s/%.3d%s",Work_dir,current_area,NEWSTXT);
	remove( area_name ); /* just in case */
	
	/*
	** Processing Mail
	*/
	message_cnt=0;

	if((Get_mail==ASK) && (Index_only==UNSET))
	{
		printf("Do you want to save your mail (y/N) ? ");
		fflush(stdout);
		fgets(templine,80,stdin);
		if ((templine[0]=='y') || (templine[0]=='Y'))
			Get_mail=SET;
	}

	if((Get_mail==SET) && (Index_only==UNSET))
	{
		if ((pos=getenv("MAIL"))==NULL)
		{
			fprintf(stderr,"Error: $MAIL not defined\n");
			return(1);
		}
		strcpy (mailbox,pos);
		if ((mbox=fopen(mailbox,"r"))!=NULL)
		{
			fclose(mbox);
			add_file(area_name,mailbox,0); /* No rnews header */

			/*
			** Update the AREAS file 
			** This version does not create an index file 
			*/
			fprintf (areas,"001\tPrivate Mail\t%cn\n",mailbox_type);
			message_cnt=0;

			if (Delete_mailbox==UNSET)
			{
				/* Move the mailbox to the .oldmail file */
				sprintf(templine,"%s/.oldmail",home);
				add_file(templine,mailbox,0);
				printf("Your mailbox has been appended to file %s\n",templine);
			}
			/* Create an empty mailbox */
			remove(mailbox);
			fw=fopen(mailbox,"w"); 
			fclose(fw);

			current_area++;
			sprintf (area_name,"%s/%.3d%s",Work_dir,current_area,NEWSTXT);
			remove( area_name ); /* just in case */
		}
	}

	/*
	** Processing News
	*/
	if (Mark_read==ASK)
	{
		printf("Do you want to mark saved articles as read (y/N) ? ");
		fflush(stdout);
		fgets (templine,80,stdin);
		if ((templine[0]=='y') || (templine[0]=='Y'))
			Mark_read=SET;
	}

	if (Mark_read==SET)
	{
		/* Create a backup .newsrc */
		sprintf( templine,"%s/%s.bak", home, NEWSRC);
		remove( templine );

		add_file( templine, newsrc, 0 );

		/* We create the new .newsrc */
		if((rc2=fopen(nrc2,"w"))==NULL)
		{
			fprintf (stderr,"Unable to create [%s] file\n",nrc2);
			return(1);
		}
	}

	/*
	** Main Loop
	*/

	/* 
	** For each line in the .newsrc
	*/
	while( (retv=lfgets(&nrcline,&nrcsize,rc)) == LFGETS_OK ) 
	{
		if (Grpmax && (current_area > Grpmax))
		{
			if( !nowrite ) /* Prints only the msg once */
				printf("\nMaximum number of groups reached (%d)\n",Grpmax);
			nowrite = 1;
		}

		/* 
		** Selects only the subscribed newsgroups 
		*/
		if (!nowrite && (pos=strchr(nrcline,':'))!=NULL)
		{
			if (nrcline[strlen(nrcline)-1]=='\n')
				nrcline[strlen(nrcline)-1]='\0';

			offset = 0;
			message_cnt=0; /* reset the message counter */
			i=0;
			while (nrcline[i]!=':')
			{
				newsgroup[i]=nrcline[i];
				i++;
			}
			newsgroup[i]='\0';

			/*
			** Save the list of read articles and
			** Get the number of the last one
			** the newsrc line looks like :
			** groupname: value-value,...,value
			*/
			pos++;
			while(*pos==' ') /* Skip the space(s) */
				pos++;
			/* 
			** pos points to either the first character of the article list,
			** or to the \0 string terminator. 
			*/
			list_read = pos;

			pos=nrcline+strlen(nrcline)-1; /* Points to the latest char */
			while (isdigit(*pos))
				pos--;
			pos++; /* Skip the - , or : character */
			last=atoi(pos);
			biggest=last;

			strcpy (ngrp2,newsgroup);
#ifndef USE_NNTP
			dot2slash(newsgroup);
			sprintf (newsdir,"%s/%s",SPOOL,newsgroup);
#endif

      if ((!nntp && chdir(newsdir))
#ifdef USE_NNTP
		  || /* directory doesnt exist */
      (nntp && nntpsetgroup(ngrp2)!=OK) /* unknown group */
#endif
      )
			{
				/*
				** This newsgroup does not exist any more
				** It may be a good idea to remove it from the .newsrc
				*/
				fprintf (log,"%s : unknown\n",ngrp2);
			}
			else if ( !nntp )
			{

				/*
				** The newsgroup exists and we are reading it 
				** by conventional methods
				*/

				/* 
				** Look for unread file 
				*/
				if ( (rep=opendir(newsdir))==NULL)
					fprintf (log,"DIR : %s not readable\n",newsdir);
				else while ((ficdir=readdir(rep))!=NULL)
				{
					sprintf (message,"%s/%s",newsdir,ficdir->d_name);

					message_no = atoi(ficdir->d_name);


					if (message_no && is_unread(message_no,list_read,last))
					{
						/*
						** We have found a new article.
						** Look if it's number is bigger than the biggest
						*/
						if (Msgmax && (total_msg >=Msgmax))
						{
							if( !nowrite )
								printf("\nMaximum number of messages reached (%d)\n",Msgmax);
							nowrite = 1;
						}
						
						if( !nowrite )
						{
							if (message_no > biggest)
								biggest=message_no;
							message_cnt++;
							total_msg++;
							/* Only display the group if there are messages */
							if (message_cnt==1)
								printf ("\n%s:",ngrp2);

							if ((message_cnt%10)==0)
							{
								printf ("."); /* show the message progression */
								fflush(stdout);
							}
							/*
							** Add the message to the temporary file
							** Or create the index
							*/
							if (Do_index==SET)
								index_message(current_area,message,offset,Encoding,Index_type);
							if (Index_only==UNSET)
								offset += add_file (area_name,message,1);	

						}
					}
				}
				closedir(rep);
			}

#ifdef USE_NNTP
			else if ( nntp )
			{
				/*
				** The newsgroup exists and we are reading it
				** by NNTP
				*/

				for( message_no=nntp_first; message_no<=nntp_last; 
					message_no++ )
				if (message_no && is_unread(message_no,list_read,last))
				{

					sprintf (message,"%s/%d", Work_dir, message_no );

					if ( nntpmsgishere( ngrp2, message_no, message ) )
					{

						/*
						** We have found a new article.
						** Look if it's number is bigger than the biggest
						*/
						if (Msgmax && (total_msg >=Msgmax))
						{
							if( !nowrite )
								printf("\nMaximum number of messages reached (%d)\n",
								Msgmax);
								nowrite = 1;
						}

						if( !nowrite )
						{
							if (message_no > biggest)
							biggest=message_no;
							message_cnt++;
							total_msg++;
							/* Only display the group if there are messages */
							if (message_cnt==1)
							printf ("\n%s:",ngrp2);

							if ((message_cnt%10)==0)
							{
								printf ("."); /* show the message progression */
								fflush(stdout);
							}
							/*
							** Add the message to the temporary file
							** Or create the index
							*/
							if (Do_index==SET)
								index_message(current_area,message,offset,Encoding,Index_type);
							if (Index_only==UNSET)
								offset += add_file (area_name,message,1);   

						}
					}
					remove( message );
				}
			}
#endif

			if (message_cnt)
			{
				/*
				** Write an entry in the AREAS list file
				*/
				if(get_description(ngrp2,templine)==NULL)
					strcpy (templine,"Unknown");
				fprintf (areas,"%.3d\t%s\tu%c\t%s\t%d\n",current_area,
					ngrp2,
					(Do_index==SET) ? 'c' : 'n',
					templine,message_cnt);
				current_area++;
				sprintf (area_name,"%s/%.3d%s",
					Work_dir,current_area,NEWSTXT);
				remove( area_name ); /* Just in case */
			}

			if (Mark_read==SET)
			{
				fprintf (rc2,"%s: 1-%d\n",ngrp2,biggest);
			}

		}
		else if (Mark_read==SET)
		{
			/* Just copy the line as it is */
			fprintf(rc2,"%s",nrcline);
		}
	}
	if(retv == LFGETS_ERR) 
	{
		fprintf(stderr,
			"\nWarning - Malloc failure while processing file %s\n", NEWSRC );
		exit(1);
	}

group_done:
	fclose(rc);
	if (Mark_read==SET)
	{
		fclose(rc2);
		rc=fopen(newsrc,"w");
		if (rc!=NULL)
		{
			/*
			** Rewrites the .newsrc
			*/
			printf ("\nRewriting .newsrc\n");
			rc2=fopen(nrc2,"r");
			while(fgets(nrcline,160,rc2)!=NULL)
				fprintf(rc,"%s",nrcline);
			fclose(rc2);
			fclose(rc);
			remove(nrc2);
		}
		else
			fprintf (log,"%s : Unable to rewrite .newsrc \n",getenv("LOGNAME"));
	}

	if (log != stderr)
		fclose (log);
	fclose (areas);
	printf ("\n%d messages processed\n",total_msg);

	/*
	** Now compress the work files
	*/
	chdir(Work_dir);

	/* 
	** Check that the compress program exists 
	** and that we can create the packet file
	*/
	sprintf(templine,"%s.%s",Packet_name,Arctype);

	if( (rc=fopen(templine,"w"))==NULL)
		fprintf(stderr,"Unable to compress *.MSG and AREAS files\n");
	else
	{
		fclose(rc);
		remove(templine);

		sprintf (templine,"%s %s.%s *.MSG *.IDX %s",
			Compress,Packet_name,Arctype,AREALIST);
		
		if(system(templine))
			fprintf(stderr,"Compress program returned an error, work files not removed\n");
		else
		{
			sprintf (templine,"%s *.MSG *.IDX", BIN_RM );
			system(templine);
			remove(AREALIST);
		}
		printf ("\nFile %s.%s has been created\n",Packet_name,Arctype);
	}
#ifdef USE_NNTP
    if ( nntp )
        nntp_finish();
#endif
	return (0); /* Normal End */
}

void dot2slash(line)
char *line;
{
	if (line==NULL)
		return;
	while (*line)
	{
		if (*line=='.')
			*line='/';
		line++;
	}
}

/* 
** add_file(f1,f2) append file f2 behind file f1 
** and return the number of bytes written
*/
unsigned long add_file (f1,f2,headermode)
char *f1,*f2;
int headermode;
{
	char header[80];
	FILE *df1,*df2;
	int c;
	unsigned long length;
	struct stat buf;

	if ( ((df1=fopen(f1,"a"))==NULL) || ((df2=fopen(f2,"r"))==NULL) )
	{
		fprintf (stderr,"Unable to open a file [%s] or [%s]!\n",f1,f2);
		return (0);
	}

	if (headermode)
	{
		/* Adding a "#! rnews xxxx" line as header */
		stat(f2,&buf);
		length=(unsigned long)buf.st_size;
		sprintf (header,"#! rnews %ld\n",length);
		length += strlen(header);
		fprintf (df1,"%s",header);
	}

	while ((c=fgetc(df2))!=EOF)
		fputc(c,df1);

	fclose(df1);
	fclose(df2);
	return(length);
}

/*
** Get the descrition of the newsgroup
** or return a null pointer if it cannot be found
*/
char *get_description(group,text)
char *group;
char *text;
{
	char line[255],*pos;
	int n=strlen(group);
	FILE *fr;
	if ((fr=fopen(GROUPS,"r"))==NULL)
		return(NULL);
	while (fgets(line,255,fr)!=NULL)
	{
		line[strlen(line)-1]='\0';
		if (strncmp(line,group,n)==0)
		{
			pos=line+n; /* Points to the tab */
			if ((*pos=='\0')||(*(pos+1)=='\0'))
				return(NULL);
			while ((*pos=='\t')||(*pos==' '))
				pos++;
			strcpy (text,pos);
			return(text);
		}
	}
	fclose( fr );
	return(NULL);
}

/*
** Return 1 if the article of number message_no is either greater than the
** last read or not in the list of read articles
*/
#ifdef __STDC__
int is_unread(int message_no, char *list_read, int last)
#else
int is_unread(message_no,list_read,last)
int message_no,last;
char *list_read;
#endif
{
    int begin,end,value;
    char orig, *pos1, *pos;
    if ((message_no > last) || (list_read[0]=='\0'))
    	return(1);

    pos=list_read;
    orig = 0;

    while (*pos) {
    	/* Read the first number */
    	pos1=pos;
    	while (isdigit(*pos))
    		pos++;
    	switch(*pos) {
    		case ',': /* A single number */
    		case '\0': /* End of string */
    			orig = *pos;
    			value=atoi(pos1);
    			if (message_no == value)
    				return(0); /* This article has been read */
    			if (message_no < value)
    				return(1); /* Lower than the lowest read */
    			break;

    		case '-': /* A range */
    			begin = atoi(pos1);
    			if (message_no == begin)
    				return(0); /* This article has been read */
    			if (message_no < begin)
    				return(1); /* Lower than the lowest read */
    			/* Read the second number */
    			pos++;
    			pos1=pos;
    			while(isdigit(*pos))
    				pos++;
    			orig = *pos;
    			end = atoi(pos1);
    			if (message_no < begin)
    				return(1);
    			if (message_no <= end)
    				return(0);
    			break;


    		default : /* Unknown char */
    			orig='1';
    			fprintf(stderr,"Warning, char <%c> unrecognised in .newsrc\n",*pos);
    			break;
    	}
    	if (orig)
    		pos++;
    }
    return(1); /* Unread */
}


char *upperit(line)
char *line;
{
	char *pos=line;
	while (*pos)
	{
		*pos=toupper(*pos);
		pos++;
	}
	return (line);
}


/* 
** Get a token from the token list out of the line
** update pos as the position in the line after the token
** and return the token number if found, -1 else.
*/
#ifdef __STDC__
int get_token(char *line,char **tlist,char **pos)
#else
int get_token(line,tlist,pos)
char *line, **tlist,**pos;
#endif
{
	int i=0,l,ll;
	char *c;

	/* search for the "=" separator */
	c=strchr(line,'=');
	if (c==NULL)
		return (-1); 
	ll=c-line; /* Get line length */
	*c='\0';
	/* *pos points to the first non blank character after the = */
	*pos=c+1; 
	while ((**pos==' ')||(**pos=='\t'))
		*pos += 1;
	upperit(line);
	while (tlist[i][0])
	{
		l=strlen(tlist[i]);
		if((l==ll) && (strcmp(line,tlist[i])==0))
			return(i);
		i++;
	}
	return(-1); /* failure */
}

/*
** Open the .getnewsrc file and read the config 
*/
void read_config()
{
	FILE *fr;
	int n;
	char line[80],old_line[80],*pos,rcfile[80];
	static char *tlist[17]={
		"MSGMAX",
		"GRPMAX",
		"MMDF",
		"USENET_ONLY",
		"MAIL_ONLY",
		"DELETE_MAILBOX",
		"GET_MAIL",
		"MARK_READ",
		"COMPRESS",
		"PACKET_NAME",
		"WORK_DIR",
		"ARCTYPE",
		"DO_INDEX",
		"INDEX_ONLY",
		"INDEX_TYPE",
		"ENCODING",
		""};


	/* Default */

	/* 
	** The maximum number of articles and of newsgroups
	** If UNSET, no limit
	*/

	Grpmax = Msgmax = UNSET; 

	/*
	** The following fields accept values SET and UNSET
	*/
	Mmdf = Usenet_only = Mail_only = Delete_mailbox = Index_only = UNSET;
	Do_index = SET;

	/*
	** The following field accept values SET, UNSET and ASK
	*/
	Get_mail = Mark_read = ASK;
	Index_type = 'c'; /* cnews type index */
	Encoding = 'u'; /* rnews */

	Compress[0]='\0';
	strcpy(Arctype,"zip");
	strcpy(Packet_name,PACKETNAME);
	sprintf(Work_dir,"%s/News",getenv("HOME"));

	sprintf(rcfile,"%s/.getnewsrc",getenv("HOME"));
	if ((fr=fopen(rcfile,"r"))==NULL)
		return; /* No config file, use default values */

	while( fgets( line, 80, fr ) != NULL )
	{
		strcpy( old_line, line );

		/* Ignore comments and blank lines */
		if( line[0] != '#' && line[0] != '\n' ) 
		{
			if (line[strlen(line)-1]=='\n') /* Remove end CR */
				line[strlen(line)-1]='\0';

			switch (get_token(line,tlist,&pos))
			{
				case 0: /* msgmax */
					Msgmax = atoi(pos);
					break;
				
				case 1: /* grpmax */
					Grpmax = atoi(pos);
					break;

				case 2: /* mmdf */
					if( (n = su_get( old_line, pos )) != -1 )
						Mmdf = n;
					break;

				case 3: /* Usenet Only */
					if( (n = su_get( old_line, pos )) != -1 )
						Usenet_only = n;
					break;

				case 4: /* Mail Only */
					if( (n = su_get( old_line, pos )) != -1 )
						Mail_only = n;
					break;

				case 5: /* Delete Mailbox */
					if( (n = sua_get( old_line, pos )) != -1 )
						Delete_mailbox = n;
					break;

				case 6: /* Get Mail */
					if( (n = sua_get( old_line, pos )) != -1 )
						Get_mail = n;
					break;

				case 7: /* Mark Read */
					if( (n = sua_get( old_line, pos )) != -1 )
						Mark_read = n;
					break;

				case 8: /* Compress */
					exp_env( strcpy(Compress,pos) );
					break;

				case 9: /* Packet Name */
					exp_env( strcpy(Packet_name,pos) );
					break;

				case 10: /* Work dir */
					exp_env( strcpy(Work_dir,pos) );
					break;

				case 11: /* Arc type */
					exp_env( strcpy(Arctype,pos) );
					break;

				case 12: /* Get index */
					if( (n = su_get( old_line, pos )) != -1 )
						Do_index = n;
					break;

				case 13: /* Get index */
					if( (n = su_get( old_line, pos )) != -1 )
						Index_only = n;
					break;

				case 14: /* Index type */
					Index_only = *pos;
					break;

				case 15: /* Encoding */
					Encoding = *pos;
					break;

				default:
					fprintf(stderr,"Line <%s> has no known keyword\n",old_line);
					break;
			}
		}
	}
	fclose(fr);
}

int su_get(line,pos)
char *line;
char *pos;
{
	switch( *pos ) 
	{
		case 'Y':
		case 'y':
		case 'S':
		case 's':
			return(SET);

		case 'N':
		case 'n':
		case 'U':
		case 'u':
			return(UNSET);

		default:
			fprintf(stderr, "Error in config file: Line <%s> ",line);
			fprintf(stderr, "should be either SET or UNSET\n");
			break;
	}
	return( -1 );
}

int sua_get(line,pos)
char *line;
char *pos;
{
	switch( *pos ) 
	{
		case 'Y':
		case 'y':
		case 'S':
		case 's':
			return(SET);

		case 'N':
		case 'n':
		case 'U':
		case 'u':
			return(UNSET);

		case 'A':
		case 'a':
			return(ASK);

		default:
			fprintf(stderr, "Error in config file: Line <%s> ",line);
			fprintf(stderr, "should be either SET, UNSET, or ASK\n");
			break;
	}
	return( -1 );
}

void index_message(area,message,offset,encoding,index_type)
int area,encoding,index_type;
char *message;
unsigned long offset;
{
	char From[80], Subject[80], Date[80], temp[80];
	char line[512], Ref[80], Id[80], *pos, Sel[40];
	int i=0, line_no=0,headermode=1,enc_size=0;
	unsigned long bytes = 0;
	FILE *fr,*fw;
	struct stat buf;

	if( index_type == 'n' )
		return; /* nothing to do */

	From[0] = Subject[0] = Date[0] = Ref[0] = '\0';

	sprintf(temp,"%s/%.3d.IDX",Work_dir,area);

	if ( ((fw=fopen(temp,"a"))==NULL) || ((fr=fopen(message,"r"))==NULL) )
	{
		fprintf (stderr,"Unable to open a file [%s] or [%s]!\n",temp,message);
		return;
	}

	stat( message, &buf );
	bytes = buf.st_size;

	switch( encoding )
	{
		case 'u':
			/* Adding a the size of "#! rnews xxxx" line to the offset */
			sprintf( temp, "#! rnews %ld\n", bytes);
			offset += strlen( temp );
			break;

		case 'm':
			sprintf( temp, "From getnews\n" );
			offset += strlen( temp );
			break;

		case 'M':
			offset += 5;
			break;

		case 'b':
		case 'B':
			offset += 4;
			break;
	}

	while(fgets(line,512,fr)!=NULL)
	{
		line[511]='\0'; /* Just in case */
		if (line[strlen(line)-1]=='\n')
			line[strlen(line)-1]='\0';
		if (headermode)
		{
			if (line[0]=='\0')
				headermode=0;
			else
			{
				if (strncmp(line,"From: ",6)==0)
					strcpy(From,line+6);
				else if (strncmp(line,"Subject: ",9)==0)
					strcpy(Subject,line+9);
				else if (strncmp(line,"Date: ",6)==0)
					strcpy(Date,line+6);
				else if (strncmp(line,"References: ",12)==0)
					strcpy(Ref,line+12);
				else if( strncmp( line, "Message-ID: ", 12 ) == 0 )
					strcpy( Id, line+12 );
			}
		}
		else
			line_no++;
	}
	fclose(fr);

	/* Find the last part of the path for the mesg Selector */
	pos=strrchr(message,'/');
	if (pos==NULL)
		strcpy(Sel,message);
	else
		strcpy(Sel,pos+1);

	switch( index_type )
	{
		case 'c':
			fprintf(fw,"%ld\t%s\t%s\t%s\t%s\t%s\t%ld\t%d\t%s\n",
				offset,Subject,From,Date,Id,Ref,bytes,line_no,Sel);
			break;

		case 'C':
			/* Extract the author name */
			if( (pos=strchr( From, '<' )) != NULL )
			{
				while( *pos && *pos!='>' )
					pos++;
				if( !*pos )
					strcpy( temp, pos+1 );
				else
					strcpy( temp, From );
			}
			else if(  (pos=strchr( From, '(' )) != NULL )
			{
				pos++;
				while( *pos && *pos!=')' )
					temp[i++] = *pos;
				temp[i]='\0';
			}
			else
				strcpy( temp, From );

			fprintf(fw,"%ld\t%s\t%s\t%s\t%ld\t%d\t%s\n",
				offset,Subject,temp,Date,bytes,line_no,Sel);
			break;

		case 'i':
			/* Not supported yet */
			break;
	}

	fclose(fw);
}

/*
**      exp_env()
**
** Expand any environment variables in input string.
** Put result back into input string.
** Input string must be big enough to hold the expanded expression.
**
** Returns 0 (FALSE) if any specified environment variables are not defined;
** Else returns 1 (TRUE).
*/
#ifdef __STDC__
int exp_env(char *s)
#else
int exp_env(s)
char *s;
#endif
{
    char *in, *out, *cptr, *env_val;
    int c, cv, ret_val;
    char env_name[30];
    char *temp;

    if(strchr(s,'$') == NULL)
    	return(1);

    temp = (char *)malloc((unsigned)(strlen(s)+1));
    if(temp == NULL) 
		{
    	fprintf(stderr, "Error - function exp_env: Malloc failed.\n");
    	return(0);
    }

    strcpy(temp,s);
    in = temp;
    out = s;
    ret_val = 1;

    while( c=*in++) 
		{    /* While there are characters */
    	if(c != '$') 
			{
    		/* regular part of string */
    		*out++ = (char)c;
    	}
    	else 
			{
    		/* Beginning of environment variable */
    		/* Pick off the name of environment variable */
    		cptr = env_name;
    		while( (c=*in++) && ( isalnum(c) || c == '_') )
    			*cptr++ = (char)c;
    		*cptr = '\0';
    		in--;
    		/* Get value */
    		env_val = getenv(env_name);
    		if(env_val == NULL)
    			ret_val = 0;
    		else 
				{
    			/* copy value into output string */
    			cptr = env_val;
    			while(cv=*cptr++)
    				*out++ = (char)cv;
    		}
    		if(c == '\0') break;  /* End of input string */
    	}
    }

    *out = '\0'; /* Terminate resulting string. */

    free((char *)temp);  /* Release malloced space */

    return(ret_val);
}

/*
**  Function:
**
**       int lfgets(char *s, int *ns, FILE *stream)
**
**  Gets an arbitrarily-long line from an input stream. The terminating
**  newline is returned also.
**
**  If s is NULL, then lfgets uses malloc() to get storage space of size ns
**  or larger to hold the input line. A pointer to the newly-malloced
**  space is returned in parameter s.
**  If s is not NULL, it must be a pointer obtained from a malloc-like
**  function (malloc, calloc, realloc, etc.).
**
**  Lfgets starts filling s with the input line.
**  If the line is longer than s, then lfgets uses realloc() as necessary
**  to get a large enough storage space. In this case, pointer s is changed
**  to point to the (possibly-relocated) new storage area,
**  and ns will change to reflect the new size of the storage space.
**
**  If all goes well, lfgets returns LFGETS_OK.
**  If the first character read is an end-of-file, lfgets returns LFGETS_EOF.
**  If malloc() or realloc() fail, then lfgets LFGETS_ERR.
*/

#ifdef __STDC__
int lfgets(char **s, int *ns, FILE *stream)
#else
int lfgets(s, ns, stream)
char **s;
int *ns;
FILE *stream;
#endif
{
    int c, i;
    int maxch;

    if(*s == NULL) {
    	/* malloc some space to start with */
    	*ns = 256;
    	*s = (char *)malloc((unsigned)*ns);
    	if(*s == (char *)NULL)
    		return(LFGETS_ERR);
    }

    i = 0;

    while(1) {
    	maxch = *ns - 1;  /* maximum # of characters to read into array,
    						 not counting the terminating null. */


    	/* Read data, putting it into s. */
    	for(; i < maxch; i++) {
    		c = fgetc(stream);

    		if(c == EOF) {
    			if(i == 0)
    				/* EOF at first attempt to read */
    				return(LFGETS_EOF);
    			else {
    				/* last line of file had no \n terminator. */
    				(*s)[i+1] = '\0';  /* terminate string. */
    				return(LFGETS_OK);
    			}
    		}

    		(*s)[i] = (char)c;
    		if(c == '\n') {
    			/* we have found the end of the input line */
    			(*s)[++i] = '\0';  /* terminate string. */
    			return(LFGETS_OK);
    		}
    	}

    	/* There is more of the line yet to read */
    	/* Get more storage space. */
    	*ns += 256;
    	*s = (char *)realloc(*s, (unsigned)*ns);
    	if(*s == (char *)NULL)
    		return(LFGETS_ERR);

    	/* And, repeat. */
    }

}

