
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

extern	char	*getserverbyfile();
extern	int	server_init();
extern	void	put_server();
extern	int	get_server();
extern	void	close_server();

extern int Verbose,Debug; 

#define NNTP_SERVER	"ozz"

#define FALSE 0
#define TRUE 1

#define	CHAR_INF	'1'
#define	CHAR_OK		'2'
#define	CHAR_CONT	'3'
#define	CHAR_ERR	'4'
#define	CHAR_FATAL	'5'

#define	INF_HELP	100	/* Help text on way */
#define	INF_DEBUG	199	/* Debug output */

#define	OK_CANPOST	200	/* Hello; you can post */
#define	OK_NOPOST	201	/* Hello; you can't post */
#define	OK_SLAVE	202	/* Slave status noted */
#define	OK_GOODBYE	205	/* Closing connection */
#define	OK_GROUP	211	/* Group selected */
#define	OK_GROUPS	215	/* Newsgroups follow */
#define	OK_ARTICLE	220	/* Article (head & body) follows */
#define	OK_HEAD		221	/* Head follows */
#define	OK_BODY		222	/* Body follows */
#define	OK_NOTEXT	223	/* No text sent -- stat, next, last */
#define	OK_NEWNEWS	230	/* New articles by message-id follow */
#define	OK_NEWGROUPS	231	/* New newsgroups follow */
#define	OK_XFERED	235	/* Article transferred successfully */
#define	OK_POSTED	240	/* Article posted successfully */

#define CONT_XFER	335	/* Continue to send article */
#define	CONT_POST	340	/* Continue to post article */

#define	ERR_GOODBYE	400	/* Have to hang up for some reason */
#define	ERR_NOGROUP	411	/* No such newsgroup */
#define	ERR_NCING	412	/* Not currently in newsgroup */
#define	ERR_NOCRNT	420	/* No current article selected */
#define	ERR_NONEXT	421	/* No next article in this group */
#define	ERR_NOPREV	422	/* No previous article in this group */
#define	ERR_NOARTIG	423	/* No such article in this group */
#define ERR_NOART	430	/* No such article at all */
#define ERR_GOTIT	435	/* Already got that article, don't send */
#define ERR_XFERFAIL	436	/* Transfer failed */
#define	ERR_XFERRJCT	437	/* Article rejected, don't resend */
#define	ERR_NOPOST	440	/* Posting not allowed */
#define	ERR_POSTFAIL	441	/* Posting failed */

#define	ERR_COMMAND	500	/* Command not recognized */
#define	ERR_CMDSYN	501	/* Command syntax error */
#define	ERR_ACCESS	502	/* Access to server denied */
#define ERR_FAULT	503	/* Program fault, command not performed */

/* RFC 977 defines this; don't change it. */

#define	NNTP_STRLEN	512
#define	LEN	512

/* part of following src code, is nntp_tass news reader code. */

/*--------------------------------------------------------------------*/

nntp_startup()
{

	char *server_name = NNTP_SERVER;
	int ret;
	extern char *getenv();

	ret = server_init(server_name);

	switch (ret) {
	case OK_CANPOST:
	case OK_NOPOST:
		break;

	case -1:
		fprintf(stderr, "failed to connect to server\n");
		exit(1);

	default:
		fprintf(stderr, "rejected by server, nntp error %d\n", ret);
		exit(1);
	}
}

nntp_finish() {
	close_server();
}

/*
 *  get_respcode
 *  get a response code from the server and return it to the caller
 */

int get_respcode() {
	char line[NNTP_STRLEN];

	if (get_server(line, NNTP_STRLEN) == -1) {
		fprintf(stderr, "connection to server broken\n");
		exit(-1);
	}

	return atoi(line);
}

stuff_nntp(fnam)
char *fnam;
{
	char line[NNTP_STRLEN];
	struct stat sb;
	int i;
	static FILE *fp = NULL;

	extern char *mktemp();

	strcpy(fnam, "/tmp/nntpXXXXXX");
	mktemp(fnam);

	fp = fopen(fnam, "w");
	if (fp == NULL) {
		fprintf(stderr, "stuff_nntp: can't open %s: ", fnam);
		perror("");
		return FALSE;
	}

	while (1) {

		if (get_server(line, NNTP_STRLEN) == -1) {
			fprintf(stderr, "NNTP conection to server broken\n");
			exit(-1);
		}
		if (strcmp(line, ".") == 0)
			break;			/* end of text */
		strcat(line, "\n");
		if (line[0] == '.')		/* reduce leading .'s */
			fputs(&line[1], fp);
		else
			fputs(line, fp);
	}

	fclose(fp);
	fp = NULL;

	return TRUE;
}

FILE *
nntp_to_fp() {
	char fnam[LEN];
	static FILE *fp;

	if (!stuff_nntp(fnam))
		return NULL;

	fp = fopen(fnam, "r");
	if (fp == NULL) {
		fprintf(stderr, "nntp_to_fp: can't reopen %s: ", fnam);
		perror("");
		return NULL;
	}
	unlink(fnam);

	return fp;
}

FILE *
open_art_fp(group_path, art)
char *group_path;
long art;
{
	char buf[LEN];

	sprintf(buf, "article %ld", art);

	put_server(buf);
	if (get_respcode() != OK_ARTICLE)
		return NULL;

	return nntp_to_fp();
}

/*--------------------------------------------------------------------*/

unsigned nntp_first, nntp_last;
char line[ 512 ];

nntpsetgroup( group )
char *group;
{
	int dummy, count;

	if ( Debug )
		fprintf( stderr, "nntpsetgroup( '%s', s, l );\n", group );

	sprintf(line, "group %s", group);
	put_server(line);

	if (get_server(line, NNTP_STRLEN) == -1) {
		fprintf(stderr, "connection to server broken\n");
		exit(-1);
	}

	if (atoi(line) != OK_GROUP)
		return( 0 );

	if ( Debug )
		fprintf( stderr, "line='%s'\n", line );

	sscanf(line,"%d %d %d %d", &dummy, &count, &nntp_first, &nntp_last);

	return( 1 );
}

nntpmsgishere( group, message_no, message )
char *group;
int message_no;
char *message;
{
	FILE *fr,*fw;

	if ( Debug )
		fprintf( stderr, "nntpmsgishere( '%s', %d, '%s' );\n",
			group, message_no, message );

	fr = open_art_fp( group, (long)message_no );
	if ( fr==NULL )
		return( 0 );

	fw = fopen( message, "w" );
	if ( !fw )
		{
		fclose (fr);
		return( 0 );
		}

	while ( fgets( line, sizeof(line), fr ) != NULL )
		fwrite( line, 1, strlen(line), fw );

	fclose( fr );
	fclose( fw );
	return( 1 );
}

/*--------------------------------------------------------------------*/
