/*
** Postreply.c : Posting mail and messages from SOUP reply packet
** Author      : Philippe Goujard
** Started     : 08/04/93
** Version     : 1.4
*/

#include <stdio.h>
#include <string.h>

/* Post article with partial header */

/*
** If you use the NNTP inews, you need to set up the server name
** either by creating /usr/lib/news/server
** or by declaring NNTPSERVER in your environment
*/

#ifdef USE_NNTP
# define INEWSCMD "/usr/local/src/tools/nntp/inews/inews"
#else
# define INEWSCMD "inews -h" 
#endif

#define MAILCMD "mail < "
#define TMP "/tmp"
#define MMDF_SEPARATOR "\001\001\001\001"
#define MAIL_SEPARATOR "From "

union converter
{
	unsigned long l;
	unsigned char c[4];
} Conv;

int is_big_endian()
{
	char testline[80];

	Conv.l=16909060; /* Should be 01 02 03 04 */
	sprintf( testline, "%d%d%d%d",Conv.c[0],Conv.c[1],Conv.c[2],Conv.c[3]);
	if( strcmp( testline, "1234" ) == 0 )
		return( 1 ); /* true */
	return( 0 );
}

void post_new_articles(msg_file, encoding, program )
char *msg_file, *program;
char encoding;
{
	FILE *fr,*fw;
	char c, *ptr, line[255],line2[255], tmpname[80];

	if ((fr=fopen(msg_file,"r"))==NULL)
		return; /* No article to post */

	sprintf(tmpname,"%s/art.%d",TMP,getpid());
	if ((fw=fopen(tmpname,"w"))==NULL)
	{
		fclose(fr);
		return;
	}
	switch( encoding )
	{
		case 'm': /* Mail format */
			fgets(line,255,fr); /* Get the From line */
			fprintf(fw,"%s",line);

			while (fgets(line,255,fr)!=NULL)
			{
				if ( strncmp( line, "From ", 5)==0)
				{
					/* Process the article */
					fclose(fw);
					sprintf( line2, "%s %s", program, tmpname);
					system( line2 );
					fw=fopen(tmpname,"w"); /* Create a new tmp file for next article */
					fprintf(fw,"%s",line);
				}
				else
					fprintf(fw,"%s",line);
			}
			/* Process the last article */
			fclose(fw);
			sprintf (line,"%s %s",program,tmpname);
			system(line);
			break;

		case 'M': /* MMDF format */
			fgets(line,255,fr); /* Get the CtrlAs line */

			while (fgets(line,255,fr)!=NULL)
			{
				if ( strncmp( line, MMDF_SEPARATOR, 4)==0)
				{
					/* Process the article */
					fclose(fw);
					sprintf( line2, "%s %s", program, tmpname);
					system( line2 );
					fw=fopen(tmpname,"w"); /* Create a new tmp file for next article */
					fgets(line,255,fr); /* Get the CtrlAs line */
				}
				else
					fprintf(fw,"%s",line);
			}
			fclose(fw);
			break;

		case 'u':
			/* Get the rnews line */
			fgets(line,255,fr);

			while (fgets(line,255,fr)!=NULL)
			{
				if (strncmp(line,"#! rnews ",9)==0)
				{
					/* Process the article */
					fclose(fw);
					sprintf (line,"%s %s",program,tmpname);
					system(line);
					fw=fopen(tmpname,"w"); /* Create a new tmp file for next article */
				}
				else
					fprintf(fw,"%s",line);
			}
			/* Process the last article */
			fclose(fw);
			sprintf (line,"%s %s",program,tmpname);
			system(line);
			break;

		case 'b':
		case 'B':
			while( (c=fgetc( fr )) != EOF )
			{
				if( is_big_endian() ) /* Sparc and 68k processors */
				{
					Conv.c[0] = c;
					Conv.c[1] = fgetc(fr);
					Conv.c[2] = fgetc(fr);
					Conv.c[3] = fgetc(fr);
				}
				else /* Intel processors */
				{
					Conv.c[3] = c;
					Conv.c[2] = fgetc(fr);
					Conv.c[1] = fgetc(fr);
					Conv.c[0] = fgetc(fr);
				}
				/* Allocate memory */
				if( ( ptr = (char *)malloc( Conv.l * sizeof( char * ) ) ) == NULL )
				{
					fclose( fr );
					fclose( fw );
					remove(tmpname);
					return;
				}

				fread( ptr, 1, Conv.l, fr );
				fwrite( ptr, 1, Conv.l, fw );
				/* Process the article */
				fclose(fw);
				sprintf (line,"%s %s",program,tmpname);
				system(line);
				fw=fopen(tmpname,"w"); /* Create a new tmp file for next article */
				free( ptr );
			}
			fclose(fw);
			break;

	}
	fclose(fr);
   	remove(tmpname);
}

main()
{
	FILE *fr;
	int ln=0;
	char line[80],*p,message[20],program[80];

	if ((fr=fopen("REPLIES","r"))==NULL)
	{
		fprintf(stderr,"Nothing to post\n");
		return;
	}
	while(fgets(line,80,fr)!=NULL)
	{
		ln++;
		if ((p=strchr(line,'\t'))==NULL)
		{
			fprintf(stderr,"Error in REPLIES, no prefix in line %d\n",ln);
			return;
		}
		*p='\0';
		sprintf(message,"%s.MSG",line);
		strcpy(line,p+1);
		if ((p=strchr(line,'\t'))==NULL)
		{
			fprintf(stderr,"Error in REPLIES no reply kind in line %d\n",ln);
			return;
		}
		*p='\0';
		p++; /* Get the encoding type */
		if (strcmp(line,"mail")==0)
			strcpy( program, MAILCMD );
		else if (strcmp(line,"news")==0)
			strcpy( program, INEWSCMD );
		else
		{
			fprintf(stderr,"Error : reply kind invalid in line %d\n",ln);
			return;
		}
		post_new_articles(message, *p, program);
	}
	fclose(fr);
}
