/*
** SLNR          : Simple Local News Reader
** Author        : P.Goujard
** Started       : 22/01/93
** Version       : 2.13
** Last modified : 04 Nov 1993
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#ifndef NOSTAT
# include <sys/stat.h>
#endif
#include <ctype.h>  /* for toupper */
#include <stdlib.h> /* For getenv */
#include "colour.h"

#ifdef ATARI
#include <mintbind.h>
#include <vt52.h>
#endif

#ifdef __OS2__
#include <io.h>
#include <conio.h>
#endif

#ifndef ATARI
#include <fcntl.h>		/* MS-DOS *does* need fcntl.h, but not termio.h	*/
#ifndef __MSDOS__
#ifndef __OS2__
#include <unistd.h>
/*
** This is used for putting the terminal in raw mode
** Under dos we don't need that since getch() returns directly
*/
#include <termio.h>
#endif
#endif
#endif

#define MMDF_SEPARATOR "\001\001\001\001"
#define MAIL_SEPARATOR "From "
#define REWIND     1
#define SET        1
#define UNSET      0
#define ASK        2
#define COMMAND_NO 11 /* Number of commands supported in SOUP12 */

#define NO_HEADER      1L
#define GET_SUBJECT    2L
#define GET_ID         4L
#define GET_FROM       8L
#define GET_REPLYTO   16L
#define GET_FOLLOWUP  32L
#define GET_NEWSGROUP 64L

#define INITFILE "slnr.ini"

#ifdef __MSDOS__
#  define PAGER "more <"
#  define EDIT "q"
#  define COMPRESS "pkzip"
#  define MAXART 500 /* 1000;	Originally 500	*/
#  define MAXGRP 100 /* 500;	Originally 100	*/
#  define UNIXR  "rb"
#  define UNIXW  "wb"
#  define UNIXA  "ab"
#else
#  define UNIXR  "r"
#  define UNIXW  "w"
#  define UNIXA  "a"

#  ifdef ATARI
	 char PAGER[64];
#    define EDIT "???" /* To be defined by atari guru */
#    define COMPRESS "stzip"  /* To be defined by atari guru */
#    define MAXART 1000
#    define MAXGRP 200
#  else /* Unix */
#    define PAGER "less"
#    define EDIT "vi"
#    define COMPRESS "zip"
#    define MAXART 5000
#    define MAXGRP 1000
#   endif
#endif

/*
** Global variables
*/
static char pager_str[80],edit_str[80],compress_str[80],quote_str[80];
static int use_ansi;
int Lines=24;

struct header{
	char *subject;
	char *from;
	char *id;
	char *replyto;
	char *followup;
	char *newsgroups;
};

/*
** The idea of a color_string array is to allow several scr() call in
** one printf.
** If we don't do that printf only see the last scr() since they all
** have the same address.
*/
char Color_string[MAX_CLR][12];
int Color_no = 0;


/*
** This is not very clean : those 2 arrays should be allocated
** only when needed instead of reserving memory which maybe won't
** be used.
*/
struct arts {
	char from[20];
	unsigned long size;
	char subject[60];
	unsigned long rank;
	unsigned long ptr;              /* Pointer to start of article */
}
#ifdef __MSDOS2__	/* Create huge array if MS-DOS	*/
	huge article[MAXART];
#else
	article[MAXART];
#endif

struct grps {
	char prefix[10]; /* The .MSG file containing the articles */
	char name[40];
	char desc[80];
	unsigned int art;   /* number of articles in that group */
	unsigned long start; /* rank of 1st article in the article[] table */
	char enc_type;  /* Encoding type (u,m,M,b,B,i) */
	char idx_type;  /* Index type (n,c,C,i) */
	char msg_type;  /* New or mail (m,n,u) */
	unsigned int reply; /* Number of replies for that group, default 0 */
}
#ifdef __MSDOS2__	/* Create huge array if MS-DOS	*/
	huge group[MAXGRP];
#else
	group[MAXGRP];
#endif

unsigned int cur_grp=0, cur_art=0, old_cur_art=0, cur_line=0;
int no=0, Index_only=0;

struct commands {
	char name[40];
	int supported;
};

static char *command_tokens[COMMAND_NO + 1]={ "subscribe", 
	"unsubscribe", "catchup", "list", "sendme", "mail", "deletemail", 
	"mailindex", "newsindex", "get", "put", "" };

#define SUBSCRIBE   0
#define UNSUBSCRIBE 1
#define CATCHUP     2
#define LIST        3
#define SENDME      4
#define MAIL        5
#define DELETEMAIL  6
#define MAILINDEX   7
#define NEWSINDEX   8
#define GET         9
#define PUT        10

struct Packet_info {
	char version[10];
	char date[80];
	char hostname[80];
	char software[80];
	struct commands command_list[11];
} packet_info;

union converter
{
	unsigned long l;
	unsigned char c[4];
} Conv;

int is_big_endian()
{
	char testline[80];

	Conv.l=16909060; /* Should be 01 02 03 04 */
	sprintf( testline, "%d%d%d%d",Conv.c[0],Conv.c[1],Conv.c[2],Conv.c[3]);
	if( strcmp( testline, "1234" ) == 0 )
		return( 1 ); /* true */
	return( 0 );
}

/*
** Return a pointer to the 1st blanc character after
** the current word
*/
char *get_word( line )
char *line;
{
	char *pos=line;
	int end=0;
	
	while(1)
	{
		switch(*pos)
		{
			case ' ':
			case '\t':
			case '\0':
			case ':':
			case '=':
			case '{':
			case '}':
			case '|':
			case ',':
				return(pos);

			case '"':
				/* Look for the matching double quote */
				pos++;
				while(!end)
				{
					if (*pos=='\0') /* Oops no matching quote */
						return(pos);
					if (*pos=='"')
						end=1;
					else
						pos++;
				}
				break;
		}
		pos++;
	}
}

/*
** Returns NULL if a word cannot be found
** Otherwise returns a pointer to the end of the word and 
** update the pointer to the start of it if needed
*/
char *get_full_word( start, line )
char **start, *line;
{
	char *pos;
	*start=line;

	while((**start==' ')||(**start=='\t'))
		*start += 1;

	if (**start=='\0')
		return(NULL);
	pos=get_word(*start);
	return(pos);
}

#ifdef ATARI

static void cls (void);

int atari_res = 0;
int atari_old_colour[16];

static void initialise_atari(void)
{
	int   i;
	char *pager;

	/*
	 * Check the environment to see if a suitable pager is defined.
	 * If so, make up the PAGER format string to use it. In all other
	 * cases make up a generic call to "more".
	 */

	pager = (char*)getenv("PAGER");

	if (pager==(char*)NULL) {
		sprintf(PAGER, "more ");
	} else {
		sprintf(PAGER, "%s ", pager);
	}

	/*
	 * Pick up screen resolution (used in function "scr").
	 */

	atari_res = Getrez();
	if (atari_res==0) {
		fprintf(stderr, "Cannot run in ST-low resolution\n");
		exit(1);
	};

	/*
	 * Record the original palette values for restore later
	 */

	for (i=0; i<16; i++) atari_old_colour[i] = Setcolor(i, -1);

	/*
	 * Set up the colour palette to match up colours with the
	 * numbers used in the colour.h include file.
	 */

	(void) Setcolor(1, 0x000);		/* Black   */
	(void) Setcolor(2, 0x700);		/* Red     */
	(void) Setcolor(3, 0x070);		/* Green   */
	(void) Setcolor(4, 0x770);		/* Yellow  */
	(void) Setcolor(5, 0x007);		/* Blue    */
	(void) Setcolor(6, 0x707);		/* Magenta */
	(void) Setcolor(7, 0x077);		/* Cyan    */
	(void) Setcolor(8, 0x777);		/* White   */

	/*
	 * Now clear the screen
	 */

	 printf("%s\n", scr(1,WHITE, BLACK));
	 cls();
}

static void closedown_atari(void)
{
	int i;

	/*
	 * Restore the original colour palette
	 */
	 for (i=0; i<16; i++) (void) Setcolor(i, atari_old_colour[i]);
}

#endif

/*
** Generates a string with ansi colors
** The unix version checks the environment variable TERM
*/
#ifdef __STDC__
char *scr(int attribute,int foreground, int background)
#else
char *scr(attribute,foreground, background)
int attribute,foreground,background;
#endif
{
	char att_str[20],for_str[20],back_str[20];
	int n = Color_no;

	if (!use_ansi)
	{
		sprintf (Color_string[0],""); /* NULL string */
		return(Color_string[0]);
	}

#ifdef ATARI

	Color_string[n][0] = '\0';

	if (atari_res==2 || atari_res==5)
		return(Color_string[n]);  /* Skip colour change on mono screens */

	if (foreground!=NONE) {
		sprintf(for_str,  "%cb%c", '\033', 'a'+foreground);
		strcat(Color_string[n], for_str);
	};

	if (background!=NONE) {
		sprintf(back_str, "%cc%c", '\033', 'a'+background);
		strcat(Color_string[n], back_str);
	};


#else

#ifndef __MSDOS__
	if ( strstr((char *) getenv("TERM"),"ansi") == NULL)
	{
		Color_string[0][0] = '\0';
		return (Color_string[0]);
	}
#endif
	att_str[0] = '\0';
	for_str[0] = '\0';
	back_str[0] = '\0';
	if (attribute != NONE)
	{
		sprintf (att_str,"%d",attribute);
		if (foreground != NONE || background != NONE)
			strcat(att_str,";");
	}
	if (foreground != NONE)
	{
		sprintf (for_str,"%d",30+foreground);
		if (background != NONE)
			strcat(for_str,";");
	}
	if (background != NONE)
		sprintf (back_str,"%d",40+background);
	sprintf (Color_string[n],"%s%s%s%sm",COL_HEAD,att_str,
		for_str,back_str);
#endif
	Color_no++;
	Color_no = Color_no%MAX_CLR;
	return (Color_string[n]);
}

void cls(void)
{
#ifndef __MSDOS__
#ifdef ATARI
	printf(CLEAR_HOME); fflush(stdout);
#else
	if ( strstr((char *) getenv("TERM"),"ansi") != NULL)
		printf("\033[0;37;40m\033[2J\033H\n"); 
	else 
		system("tput clear 2>/dev/null");     /* else use clear with terminfo */
#endif
#else
		printf("\033[0;37;40m\033[2J\033[H");
		fflush(stdout);
#endif
}

/* Remove unwanted characters */
char *stripchars(line)
char *line;
{
	char newstr[255];
	char *str = line;
	int i = 0;

	if (strlen(line) == 0)
		return (NULL);

	while (*str)
	{
	 	if ((*str >= ' ') && (*str <= '~'))
			newstr[i++]=*str;
		str++;
	}
	newstr[i] = '\0';
	strcpy (line,newstr);

	return(line);
}

char *remove_cr(line)
char *line;
{
	int l;
	l = strlen(line);
	if( line[l-1] == '\n')
		line[l-1] = '\0';
	return(line);
}

/* 
** Like fgets but with a prompt and check line length 
*/
void fgetstr( prompt, line, default_value, size, fr)
char *prompt,*line,*default_value;
int size;
FILE *fr;
{
	int l=1,i;
	if (fr == stdin)
	{
		/* We only display the prompt if we are reading from stdin */
		printf ("%s%s%s ",scr(1,3,0),prompt,scr(1,1,0));
		if (default_value && *default_value)
		{
			if ((int)strlen(default_value) > size)
				default_value[size]='\0';
			printf("%s",default_value);
			l=strlen(default_value);
		}
		for (i = 0;i < size-l; i++)
			printf (".");
		printf ("\r%s%s%s ",scr(1,3,0),prompt,scr(8,7,0));
		fflush(stdout);
	}
	fflush(fr);
	if (fgets(line,size,fr) == NULL)
	{
		/* Return the default */
		strcpy(line,default_value);
		return;
	}
	line[size] = '\0';
	l = strlen(line);
	if (line[l-1] == '\n')
		line[l-1] = '\0';
	fflush (fr);
	stripchars(line);
	if (*line==0) /* Blank line */
		strcpy (line,default_value);
}

/*
 This is the core of the program, where we build the index table
 For each newsgroup we fill an entry in  the group[] array
 as well as for each article of a group.
 This make moving between groups very quick.
 However, reading an article can be slow : there is no index to point
 directly to an article, we only know the rank of the article.

 However compared to the .ndx method used by QWK readers, this is not
 dramatically slower. First of all the .dat files for QWK are bigger than
 our MSG files : each article has to be adjusted to be on an exact
 number of 128bytes block. And QWK .dat files contains header information
 that are far too restrictive.
*/

void extract_article(ntext,size,separator,rewind_type)
FILE *ntext;
unsigned long size; /* Size of the article, if provided ignore the separator */
char *separator;
int rewind_type; /* if set to REWIND, will unget the last line */
{
	unsigned int l,end_of_article, headermode;
	unsigned long bytes_read, line_cnt, file_pos;
	char *pos,*p,temp[255],line[255];

	if (size==0)
		l=strlen(separator);
	headermode=1;
	end_of_article=0;
	line_cnt=1;
	bytes_read=0;

	while (!end_of_article)
	{
		file_pos = ftell(ntext);
		pos = fgets(line,255,ntext);

		if (pos==NULL)
			end_of_article=1;
		else
		{
			/* Have we reached the end ? */
			bytes_read += strlen(line);
			if (size) /* For usenet message */
			{
				if (bytes_read>=size)
					end_of_article=1;
			}
			else if (strncmp(line,separator,l)==0)
				end_of_article = 1;

			if (!end_of_article)
			{
				if (headermode)
				{
					if (line[0]=='\n')
					{
						headermode=0; /* End of Header, start of article body */
						line_cnt=0;
					}
					line[strlen(line)-1]='\0';
					if (strncmp(line,"From: ",6)==0)
					{
						/* Is there a name between () ? */
						p=strchr(line,'(');
						if (p)
						{
							strcpy (temp,p+1);
							p=strrchr(temp,')');

							/* We need at least 4 chars in the name */
							if (p < temp+4)
								strcpy( temp, line+6);
							else
								*p='\0';
						}
						else /* Nope, just use the address as name */
							strcpy(temp,line+6);
						strncpy (article[cur_art].from,temp,16);
						article[cur_art].from[16]='\0';
					}
					else if( strncmp(line,"Subject: ",9)==0 )
					{
						char *lpos = line+9;
						while (( *lpos == ' ' ) || ( *lpos == '\t' ))
							lpos++;
						strncpy( article[cur_art].subject, lpos, 59);
						article[cur_art].subject[50] = '\0';
					}
				}
				else
					line_cnt++;
			}
		}
	}
	article[cur_art].size=line_cnt;
	if (rewind_type == REWIND)
		fseek(ntext,file_pos,0);
}

void index_all()
{
	char gline[255],line[255],*p,area_name[80];
	int line_no=0,n;
	unsigned long size, file_pos;
	FILE *nlist,*ntext;

	/* Open the files */
	if ((nlist=fopen("AREAS","r"))==NULL)
	{
		fprintf( stderr, "Unable to open AREAS\n" );
		exit(1);
	}

	/* For each newsgroup */
	while (fgets(gline,255,nlist)!=NULL)
	{
group_again:
		line_no++;
		if (gline[0]!='#') /* Ignore commented out lines */
		{
			/* Ignore comments at end of line */
			if ((p=strrchr(gline,'#'))!=NULL)
				*p='\0';
			if (gline[strlen(gline)-1]=='\n')
				gline[strlen(gline)-1]='\0';

			/*
			** Get the prefix name and open the file
			*/
			if ((p=strchr(gline,'\t'))==NULL)
			{
				fprintf (stderr, 
					"Invalid entry in the AREAS file, line %d\n",line_no);

				fprintf( stderr, "Skipping...\n" );
				if( fgets( gline, 255, nlist )==NULL )
				{
					fclose( nlist );
					return;
				}
				goto group_again;

			}
			*p='\0';
			sprintf(area_name,"%s.MSG",gline);
			if ((ntext=fopen(area_name, UNIXR))==NULL)
			{
				fprintf(stderr,"Unable to open area <%s>\n",area_name);

				fprintf( stderr, "Skipping...\n" );
				if( fgets( gline, 255, nlist )==NULL )
				{
					fclose( nlist );
					return;
				}
				goto group_again;

			}
			strcpy(group[cur_grp].prefix,gline);
			group[cur_grp].start=cur_art;
			strcpy(gline,p+1);

			/*
			** Get the group name
			*/
			if ((p=strchr(gline,'\t'))==NULL)
			{
				fprintf (stderr,"Invalid entry in the AREAS file, line %d\n",
					line_no);

				fprintf( stderr, "Skipping...\n" );
				if( fgets( gline, 255, nlist )==NULL )
				{
					fclose( nlist );
					return;
				}
				goto group_again;

			}
			*p='\0';
			strcpy (group[cur_grp].name,gline);
			p++;
			strcpy (gline,p);
			p=gline;

			/*
			** Get the encoding
			*/
			if (*p=='\0')
			{
				fprintf (stderr,"Invalid entry in the AREAS file, line %d\n",
					line_no);

				fprintf( stderr, "Skipping...\n" );
				if( fgets( gline, 255, nlist )==NULL )
				{
					fclose( nlist );
					return;
				}
				goto group_again;

			}
			group[cur_grp].enc_type=*p;
			p++;

			/*
			** Get the index type
			*/
			if (*p=='\0')
			{
				fprintf (stderr,"Invalid entry in the AREAS file, line %d\n",
					line_no);

				fprintf( stderr, "Skipping...\n" );
				if( fgets( gline, 255, nlist )==NULL )
				{
					fclose( nlist );
					return;
				}
				goto group_again;

			}

			group[cur_grp].idx_type=*p;
			p++;
			if (*p=='\0') /* No optional field */
				goto area_done;
			
			if( *p != '\t' ) /* It must be the msg_type */
				group[cur_grp].msg_type = *p;
			else
				group[cur_grp].msg_type = 'u'; /* Unknown */

			strcpy(gline,p+1);

			/*
			** Get the group description
			*/
			if ((p=strchr(gline,'\t'))==NULL)
			{
				strcpy(group[cur_grp].desc,gline);
				goto area_done;
			}
			*p='\0';
			strcpy(group[cur_grp].desc,gline);
			strcpy(gline,p+1);

#ifdef NEVER /* This may be needed in the future */
			/*
			** Get the number of articles
			*/
			if ((p=strchr(gline,'\t'))==NULL)
			{
				group[cur_grp].art = atol(gline);
				goto area_done;
			}
			*p='\0';
			group[cur_grp].art = atol(gline);
			strcpy (gline,p+1);
#endif

area_done:
			printf("\nReading %s ", group[cur_grp].name );

			/*
			** For each article
			*/
			n=0;
			file_pos = ftell(ntext);

			while(fgets(line,255,ntext)!=NULL)
			{

				if ((n%10)==0)
				{
					printf (".");
					fflush(stdout);
				}

				switch(group[cur_grp].enc_type)
				{

					case 'm': /* Mailbox format */
						/*
						** Read the From line
						*/
						if (strncmp(line,MAIL_SEPARATOR,5))
						{
							fprintf(stderr,"Line <%s> not in mail format\n",line);
							fprintf( stderr, "Skipping...\n" );
							goto art_done;
						}
						article[cur_art].ptr = file_pos;
						extract_article(ntext,0L,MAIL_SEPARATOR,REWIND);
						break;

					case 'M': /* MMDF format */
						/*
						** Read the Ctrl As line
						*/
						if (strncmp(line,MMDF_SEPARATOR,4))
						{
							fprintf(stderr,"Line <%s> not in MMDF format\n",line);
							fprintf( stderr, "Skipping...\n" );
							goto art_done;
						}
						article[cur_art].ptr = ftell(ntext);
						extract_article(ntext,0L,MMDF_SEPARATOR,0);
						break;

					case 'u': /* Usenet news format */
						/*
						** Read the rnews line
						*/
						if (strncmp(line,"#! rnews ",9))
						{
							fprintf(stderr,"Line <%s> not in rnews format\n",line);
							fprintf( stderr, "Skipping...\n" );
							goto art_done;
						}
						size = atol(line+9);
						article[cur_art].ptr = ftell(ntext);
						if( size )
							extract_article(ntext,size,"",0);
						break;
					
					case 'b': /* Binary */
					case 'B':
						fseek(ntext,file_pos,0);
						if( is_big_endian() ) /* Sparc and 68k processors */
						{
							Conv.c[0] = fgetc( ntext );
							Conv.c[1] = fgetc( ntext );
							Conv.c[2] = fgetc( ntext );
							Conv.c[3] = fgetc( ntext );
						}
						else /* Intel processors */
						{
							Conv.c[3] = fgetc( ntext );
							Conv.c[2] = fgetc( ntext );
							Conv.c[1] = fgetc( ntext );
							Conv.c[0] = fgetc( ntext );
						}
						size = Conv.l;
						article[cur_art].ptr = ftell(ntext);
						if( size )
							extract_article(ntext,size,"",0);
						break;

					case 'i': /* Index only */
						Index_only = 1;
						break;

					default:
						fprintf(stderr,"Error: Message type <%c> not supported\n",
							group[cur_grp].enc_type);
							fprintf( stderr, "Skipping...\n" );
							goto art_done;
				}
				article[cur_art].rank=n;
				/*
				** Here should be the code to malloc() a new article
				** For the moment we just check that we are not going
				** further than the boundary
				*/
				if (cur_art>=MAXART)
					return;
				cur_art++;
				n++;
art_done:
				file_pos = ftell(ntext);
			}
			group[cur_grp].art=n;
			/*
			** Here should be the code to malloc() a new group
			** For the moment we just check that we are not going
			** further than the boundary
			*/
			fclose(ntext);
			if (cur_grp>=MAXGRP)
				return;
			cur_grp++;
		}
	}
	fclose (nlist);
}

void disp_f(ptr)
struct arts *ptr;
{
	printf ("%s%3ld %s%-16s %s%4ld %s%-53s\n",
		(ptr->rank+1)==no ? scr(1,GREEN,0) : scr(1,YELLOW,0), ptr->rank+1,
		scr(1,MAGENTA,0),ptr->from,
		scr(1,YELLOW,0),ptr->size+1,
		scr(1,CYAN,0),ptr->subject);
}

void show_header()
{
	int n=group[cur_grp].art;
	char desc[80];

	cls();
	sprintf( desc, "%s  <%s>", group[cur_grp].name, group[cur_grp].desc );
	desc[59] = '\0';

	printf( "%sGroup %s%-60s %s%d %sarticle%c\n",
		scr( 1, GREEN, 0 ), scr( 1, CYAN, 0 ), desc,
		scr(1,YELLOW,0),n,scr(1,GREEN,0),(n>1) ? 's' : ' ');

	printf ("    %sFrom             Lines         Subject\n",scr(1,CYAN,0));
	printf ("%s============================================================================\n",scr(1,GREEN,0));
}

char ch_get(void)
{
#ifdef NORAW
	char c;
	fflush(stdin);
	c=fgetc(stdin);
	fflush(stdin);
	return(c);
#else
#ifdef ATARI
	return((char)Cnecin());
#else
#ifdef __MSDOS__
	return (getch());
#else
#ifdef __OS2__
	return (getch());
#else
	int file;
	char return_char;
	struct termio newterm, oldterm;
	file = open("/dev/tty", O_RDONLY);
	if (file == -1)
		return (0);
	ioctl(file,TCGETA,&oldterm);
	memcpy (&newterm,&oldterm,sizeof(struct termio));
	newterm.c_cc[4] = 1;
	newterm.c_cc[5] = 0;
	/* newterm.c_cflag = (B9600|CS8|CREAD); */
	newterm.c_lflag = 0;
	newterm.c_iflag = IGNBRK;
	newterm.c_oflag = 0;
	ioctl (file,TCSETAF,&newterm);
	fflush(stdout);
	read(file,&return_char,1);
  ioctl (file,TCSETAF,&oldterm);
  close(file);
	return (return_char);
#endif
#endif
#endif
#endif /* NORAW */
}


#ifdef __STDC__
int edit_file(char *name,char *header)
#else
int edit_file(name,header)
char *name, *header;
#endif
{
	char temp[80],temp2[80],choice;
	int end=0;
	FILE *fr;

	sprintf (temp,"%s %s",edit_str,name);
	sprintf (temp2,"%s %s",edit_str,header);
	system(temp);
	while (!end)
	{
		printf("%sP%sost, %sE%sdit_file %sH%seader_edit or %sA%sbandon : ",
			scr(1,3,0),scr(1,6,0),
			scr(1,3,0),scr(1,6,0),
			scr(1,3,0),scr(1,6,0),scr(1,3,0),scr(1,6,0));
		fflush(stdout);

		choice=ch_get();
		switch(choice)
		{
			case 'p':
			case 'P':
				/* Does the article exist ? */
				if ((fr=fopen(name,"r"))==NULL)
					return(1);
				fclose(fr);
				end = 1;
				break;

			case 'E':
			case 'e':
				system(temp);
				break;

			case 'H':
			case 'h':
				system(temp2);
				break;

			case 'a':
			case 'A':
				remove(name);
				end = 2;
		}
	}
	return( end-1 );
}

#ifdef NOSTAT
unsigned long get_length( file )
char *file;
{
	FILE *fr;
	unsigned long l=0;
	if( (fr = fopen( file, UNIXR )) == NULL )
		return(0);
	while( fgetc(fr) != EOF )
		l++;
	fclose( fr );
	return(l);
}
#endif

#ifdef __STDC__
unsigned long add_file (char *f1,char *f2)
#else
unsigned long add_file (f1,f2)
char *f1,*f2;
#endif
{
	char header[80];
	FILE *df1,*df2, *temp;
	int c;
	unsigned long length;
#ifndef NOSTAT
	struct stat buf;
#endif

	if ( ((df1=fopen(f1, UNIXA))==NULL) || ((df2=fopen(f2,"r"))==NULL) )
	{
		fprintf (stderr,"Unable to open a file [%s] or [%s]!\n",f1,f2);
		return (0);
	}
/*
 * This section of the code uses C's automatic text translation
 * feature to translate from what is probably MS-DOS text to Unix text.
 * You can #ifdef the code out for other platforms (although it shouldn't
 * hurt anything if left in -- it will just do an unnecessary translation
 * back to the same thing).
 *
 * C will translate \n\r to the logical \n when reading in text mode
 * to maintain compatibility.
 * We can then write the text back out in binary (UNIXW) mode, and
 * the logical \n's will be written as literal \n's.  Voila!
 */

	temp = fopen("translat.tmp", UNIXW) ;	/* temp translation file	*/

	while ((c=fgetc(df2))!=EOF)
		fputc(c,temp);

	fclose(df2) ;
	fclose(temp) ;
/*
 * At this point, there is a temporary file that has been translated
 * to Unix text.  We can now get the correct size of it and add it to the
 * packet.
 */
	temp = fopen("translat.tmp", UNIXR) ;
#ifdef NOSTAT
	length = get_len("translat.tmp" );
#else
	stat("translat.tmp", &buf);
	length=(unsigned long)buf.st_size;
#endif
	sprintf (header,"#! rnews %ld\n",length);
	length += strlen(header);
	fprintf (df1,"%s",header);

	while ((c=fgetc(temp))!=EOF)
		fputc(c,df1);

	fclose(df1);
	fclose(temp);
	remove("translat.tmp") ;
	return(length);
}


/*
** Append the content of the signature file to the file name
*/
#ifdef __STDC__
void addsig(char *name)
#else
void addsig(name)
char *name;
#endif
{
	FILE *fr,*fw;
	char line[81];

	if ((fr=fopen("sig.txt","r"))==NULL)
		return;
	if ((fw=fopen(name,"a"))==NULL)
		return;
	fprintf(fw,"\n-- \n");
	while (fgets(line,80,fr)!=NULL)
		fprintf(fw,"%s",line);
	fclose(fr);
	fclose(fw);
}

char *upperit(line)
char *line;
{
	char *pos=line;
	while (*pos)
	{
		*pos=toupper(*pos);
		pos++;
	}
	return (line);
}

char *lowerit( line )
char *line;
{
	char *pos=line;
	while (*pos)
	{
		*pos=tolower(*pos);
		pos++;
	}
	return (line);
}

int get_set_unset(line)
char *line;
{
	if( ( line[0]=='Y') || (line[0]=='y') || 
		(line[0]=='S') || (line[0]=='s'))
		return(SET);

	if( ( line[0] == 'N' ) || ( line[0] == 'n' ) || 
		(line[0]=='U') || (line[0]=='u'))
		return(UNSET);

	fprintf( stderr, "Error in config file: Line <%s>"
		"should be either SET or UNSET\n",line);
	return( -1 );
}

int get_set_unset_ask(line)
char *line;
{
	if( ( line[0] == 'Y' ) || ( line[0] == 'y' ) || 
		( line[0] == 'S' ) || ( line[0] == 's' ) )
		return(SET);

	if( ( line[0] == 'N' ) || ( line[0] == 'n' ) || 
		( line[0] == 'U' ) || ( line[0] == 'u' ) )
		return(UNSET);

	if((line[0]=='A') || (line[0]=='a'))
		return(ASK);
	fprintf(stderr,"Error in config file: Line <%s>"
		" should be either SET, UNSET or ASK\n",line);
	return( -1 );
}

/*
** Get a token from the token list out of the line
** update pos as the position in the line after the token
** and return the token number if found,
** Else returns -1 if we are at the end of the string and
** -2 for a failure.
*/

int get_token(char **pos, char *line, char **tlist)
{
	int i=0,l,ll;
	char *c,token[80];
	*pos=line;

	/*
	** c points to the first character after the token
	*/
	if ((c=get_full_word(pos,line))==NULL)
		return(-1);

	/* Copy the token */
	ll=c - *pos; 
	strncpy(token,*pos,ll);
	token[ll]='\0';
	strlwr(token);

	/* *pos points to the first non blank character after the token */
	*pos=c; 
	while ((**pos==' ')||(**pos=='\t'))
		*pos += 1;
	while (tlist[i][0])
	{
		l=strlen(tlist[i]);
		if((l==ll) && (strcmp(token,tlist[i])==0))
			return(i);
		i++;
	}
	return(-2); /* failure */
}


int get_config_token( pos, line, tlist )
char *line, **tlist,**pos;
{
	char *c, *scratch;

	/* search for the "=" separator */
	c=strchr(line,'=');
	if (c==NULL)
		return (-2);

	*pos=c+1;
	return( get_token( &scratch, line, tlist) );
}

/*
** Open the slnr.ini file and read the config for
** editor, ansi, pager, compress
*/
void read_config()
{
	FILE *fr;
	int n;
	char line[80],old_line[80],*pos;
	static char *tlist[7]={"editor",
		"pager",
		"compress",
		"ansi",
		"lines",
		"quote",
		""};

	/*
	** Default
	*/

	strcpy (edit_str, getenv("EDITOR") ? getenv("EDITOR") : EDIT);
	strcpy (pager_str, getenv("PAGER") ? getenv("PAGER") : PAGER);
	strcpy (compress_str,COMPRESS);
	strcpy (quote_str,"> ");
	use_ansi = SET;
	Lines = 24;

	if ((fr=fopen(INITFILE,"r"))==NULL)
		return; /* No config file */
	while (fgets(line,80,fr)!=NULL)
	{
		strcpy (old_line,line);
		if (line[0]!='#' && line[0]!='\n') /* Ignore comments and blank lines */
		{
			if (line[strlen(line)-1]=='\n') /* Remove end CR */
				line[strlen(line)-1]='\0';
			switch( get_config_token( &pos, line, tlist ) )
			{
				case 0: /* EDITOR */
					strcpy (edit_str,pos);
					break;

				case 1: /* PAGER */
					strcpy (pager_str,pos);
					break;

				case 2: /* COMPRESS */
					strcpy (compress_str,pos);
					break;

				case 3: /* Ansi */
					if( (n = get_set_unset(pos)) != -1 )
						use_ansi = n;
					break;

				case 4: /* Lines */
					Lines = atoi(pos);
					if (Lines<5)
						Lines = 24;
					break;

				case 5: /* Quote string */
					/* Skip the spaces */
					while ((*pos==' ') || (*pos=='\t'))
						pos++;

					if( *pos == '"')
					{
						strcpy(quote_str,pos+1);
						/* Find the matching double quote */
						pos = strrchr(quote_str,'"');
						if (pos)
							*pos = '\0';
					}
					else
						strcpy(quote_str,pos);
					remove_cr(quote_str);
					break;

				default:
					fprintf(stderr,"Line <%s> has no keyword\n",old_line);
					break;
			}
		}
	}
	fclose(fr);
}

/*
** Put the message pointed by fr into the filename and
** returns the number of lines in the header
** or 0 if an error occured
** And quote the message with the quote string
*/

int put_into_file(filename,fr,quote,flags,head)
char *filename;
FILE *fr;
char *quote;
struct header *head;
unsigned long flags;
{
	FILE *fw;
	int end=0,l,headermode=1,line_no=1,mail_flag=0;
	char separator[80],line[255],mode[5];

	strcpy( mode, "w" ); /* default */

	if(( fw = fopen( filename, "r" ) ) != NULL )
	{
		fclose( fw );
		printf( "File <%s> already exists\n", filename );
		while( !end )
		{
			printf( "[A]ppend [O]verwrite [Q]uit:" );
			fflush( stdout );
			switch( ch_get() )
			{
				case 'q':
				case 'Q':
					return(0);

				case 'a':
				case 'A':
					strcpy( mode, "a" );
					end=1;
					break;

				case 'O':
				case 'o':
					strcpy( mode, "w" );
					end=1;
					break;
			}
		}
	}

	if((fw=fopen(filename, mode))==NULL)
		return(0);

	if(group[cur_grp].enc_type=='M')
		strcpy(separator, MMDF_SEPARATOR);
	else if(group[cur_grp].enc_type=='m')
	{
		strcpy(separator, "From ");
		mail_flag = 1;
	}
	else
		strcpy(separator,"#! rnews");
	l = strlen(separator);

	fgets(line,255,fr);
	if (mail_flag)
	{
		if (!(flags & NO_HEADER))
			fprintf (fw,"%s",line);
		line_no++;
		if( fgets( line, 255, fr ) == NULL )
			return(0);
	}

	while( strncmp( line, separator, l ))
	{
		if( headermode )
		{
			if (!(flags & NO_HEADER))
				fprintf (fw,"%s",line);

			if( (flags & GET_SUBJECT) && strncmp(line,"Subject: ",9)==0)
				strcpy( head->subject, line+9);

			if( (flags & GET_ID) && strncmp(line,"Message-ID: ",12)==0)
				strcpy( head->id, line+12);

			if( (flags & GET_FROM) && strncmp(line,"From: ",6)==0)
				strcpy( head->from, line+6);

			if( (flags & GET_REPLYTO) && strncmp(line,"Reply-To: ",10)==0)
				strcpy( head->replyto, line+10);

			if( (flags & GET_NEWSGROUP) && strncmp( line, "Newsgroups: ",12)==0)
			{
				strcpy( head->newsgroups, line+12);
				head->newsgroups[254] = '\0';
				remove_cr( head->newsgroups );
			}

			if( (flags & GET_FOLLOWUP) && strncmp( line, "Followup-To: ",13)==0)
			{
				strcpy( head->followup, line+13);
				head->followup[254]='\0';
				remove_cr( head->followup );
			}

		}
		else
		{
			fprintf( fw,"%s%s",quote,line);
			line_no++;
		}

		if (line[0]=='\n') /* End of header */
			headermode=0;

		if(fgets(line,255,fr)==NULL)
			break;
	}
	fclose(fw);
	return(line_no);
}

void show_menu(tof,eof)
unsigned int tof,eof;
{
	printf ("\n%sA%sgain %sV%siew %sM%sail ",
		scr(1,3,0),scr(1,6,0),scr(1,3,0),scr(1,6,0),scr(1,3,0),scr(1,6,0));
	if (group[cur_grp].enc_type=='u')
	/* No follow-up for private mail */
		printf ("%sW%srite ",scr(1,3,0),scr(1,6,0));
	if (!eof)
		printf ("%sn%sext_pg ",
			scr(1,3,0),scr(1,6,0));
	if (!tof)
		printf ("%sp%srev_pg ",
			scr(1,3,0),scr(1,6,0));
	printf ("%sN%sext_grp %sP%srev_grp %sS%save %sH%slp %sQ%suit ",
			scr(1,3,0),scr(1,6,0),
			scr(1,3,0),scr(1,6,0),
			scr(1,3,0),scr(1,6,0),
			scr(1,3,0),scr(1,6,0),
			scr(1,3,0),scr(1,6,0));
	printf("%s", scr(1, WHITE, 0));
	fflush(stdout);
}

void display_page_again(line_no)
unsigned int line_no;
{
	int i,j;

	show_header();
	cur_art=old_cur_art;
	for (j=0;j<cur_line;j++)
	{
		disp_f(&article[cur_art]);
		cur_art++;
	}
	if (cur_line!=line_no)
	{
		for (i=cur_line; i<=line_no; i++)
			printf("\n");
	}
}

/*
** Get a clean address to be passed back to the mailer
*/
char *clean_to( to )
char *to;
{
	char *pos1, *pos2;

	/*
	** Remove the comment between ()
	*/
	if( (pos1 = strchr( to, '(' )) != NULL )
	{
		if( (pos2=strrchr(pos1,')')) != NULL )
			strcpy( pos1, pos2+1 );
		else
			*pos1 = '\0';
	}
	/*
	** Get the address between <>
	*/
	if( (pos1 = strchr( to, '<' )) != NULL )
	{
		strcpy( to, pos1+1 );
		if( (pos2 = strchr( to, '>' )) != NULL )
			*pos2 = '\0';
	}
	return( to );
}

void scan_soup_packet()
{
	FILE *fr;
	int i,n;
	char line[255], *pos_end, *pos;
	static char *command_expected[7]={ "version", "date", "hostname",
		"software", "supported", "put", "" };

	if( (fr=fopen( "INFO", "r" )) != NULL )
	{
		fclose( fr );
		sprintf( line, "%s INFO", pager_str );
		system( line );
	}

	/*
	** Set the default packet infos 
	*/
	strcpy( packet_info.version, "?.?" );
	strcpy( packet_info.date, "unknown" );
	strcpy( packet_info.hostname, "unknown" );

	for( i=0; i<COMMAND_NO; i++ )
	{
		strcpy( packet_info.command_list[i].name, command_tokens[i] );
		packet_info.command_list[i].supported = 0;
	}

	if( (fr=fopen( "COMMANDS", UNIXR )) != NULL )
	{
		while( fgets( line, 255, fr ) != NULL )
		{
			if( line[0] == '#' ) /* Ignore comments */
				continue;

			pos = line;
			remove_cr( line );

			while( isspace( *pos ) )
				pos++;

			switch( get_token( &pos_end, pos, command_expected ) )
			{
				case 0: /* version */
					strcpy( packet_info.version, pos_end );
					break;
				
				case 1: /* date */
					strcpy( packet_info.date, pos_end );
					break;

				case 2: /* hostanme */
					strcpy( packet_info.hostname, pos_end );
					break;

				case 3: /* software */
					strcpy( packet_info.software, pos_end );
					break;

				case 4: /* supported */
					pos = pos_end;
					while( (n=get_token( &pos_end, pos, command_tokens )) != -1 )
					{
						if( n == -2 )
							fprintf( stderr, "Unknown token <%s>\n", pos );
						else
							packet_info.command_list[n].supported = 1;
						pos = pos_end;
					}
					break;
				
				case 5: /* put */
					printf( "You have received the file %s\n", pos );
					break;

				default: 
					fprintf( stderr, "Unknown command <%s>\n", pos );
					break;
			}
		}
	}

}

/*
** Displays the info extracted from the COMMANDS file
*/
void option_show_info()
{
	int i;

	cls();
	printf( "SOUP version  : %s\n", packet_info.version );
	printf( "Creation date : %s\n", packet_info.date );
	printf( "Host          : %s\n", packet_info.hostname );
	printf( "Software      : %s\n\n", packet_info.software );

	for( i=0; i<COMMAND_NO; i++ )
	{
		printf( "%-20s%s\n", packet_info.command_list[i].name,
			(packet_info.command_list[i].supported == 1 ) ? "Supported" :
				"Unsupported" ); 
	}
	printf( "\n Press any key to continue\n" );
	ch_get();
}

int main()
{
	unsigned int count, line_no,i=0,j=0,end=0,eof=0;
	unsigned int reply_created=0,ret;
	unsigned int tof=1,redisplay=1,Max_grp,Max_art,mail_reply_no=0;
	char line[255],temp[80],choice,value[80],file_list[255];
	char ngrpline[255], followupline[255];
	char To[80],Replyto[80],From[80];
	char Reply[80],subjectline[80],msgidline[255];
	char art_save_name[80],shell_cmd_name[80];
	FILE *fr,*fw;
	struct header head;

#ifdef ATARI
	initialise_atari();
#endif

	read_config();

	To[0]=Replyto[0]=From[0]=art_save_name[0]=shell_cmd_name[0]='\0';
	cls();
	printf ("\n%sS%simple %sL%socal %sN%sews %sR%seader. V 1.3 (c) 1993 Philippe Goujard\n",
		scr(1,3,0),scr(1,6,0),
		scr(1,3,0),scr(1,6,0),
		scr(1,3,0),scr(1,6,0),
		scr(1,3,0),scr(1,6,0));
	/*
	** To be changed when we have another compressor
	*/
	sprintf (Reply,"REPLY.zip");
	if ((fr=fopen(Reply,"r"))!=NULL)
	{
		fclose (fr);
		printf ("\nA Reply packet already exists. Keep it (Y/n) ? ");
		fflush(stdout);
		choice=ch_get();
		if (choice=='n' || choice=='N')
			remove (Reply);
		fflush(stdin);
	}
	for (i=0; i<MAXGRP; group[i++].reply=0)
		;

	scan_soup_packet();
	index_all();

	Max_grp=cur_grp-1;
	Max_art=cur_art;   /* Not needed in this version */

	cur_art=0;
	cur_grp=0;

#ifdef INFOCOM
	/*
	** This is only done with Infocom where a user can define a profile which
	** is independant of the Unix environment. It can be easily changed to a
	** "GETENV"
	*/
	if (get_prof("LINES",value)>0)
	{
		line_no = atoi(value)-6;
		if (line_no < 7)
			line_no=18;
	}
#else
	line_no = Lines-6;
#endif
	/* Displays header */
	show_header();

	/* Displays list upto LINES - 6 */
	i=0;

	while ((cur_line<line_no) && cur_line<group[cur_grp].art)
	{
		disp_f(&article[cur_art]);
		cur_line++;
		cur_art++;
	}
	if (cur_line!=line_no)
	{
		eof=1;
		for (i=cur_line; i<=line_no; i++)
			printf("\n");
	}

	/*
	** MAIN LOOP
	*/
	while (!end)
	{
		/* Displays footer */
		if (redisplay)
			show_menu( tof, eof);

		choice=ch_get();
		redisplay=1;
		switch (choice)
		{
			case 'A':
			case 'a':
				display_page_again(line_no);
				break;

			case 'n': /* Next page */
				if (eof)
					redisplay=0;
				else
				{
					tof=0;
					show_header();
					cur_line=1;
					old_cur_art=cur_art;
					while ((cur_line<line_no) &&
						(article[cur_art].rank+1 < group[cur_grp].art))
					{
						disp_f(&article[cur_art]);
						cur_line++;
						cur_art++;
					}
					disp_f(&article[cur_art++]);
					if (cur_line!=line_no)
					{
						eof=1;
						for (i=cur_line; i<=line_no; i++)
							printf("\n");
					}
					else
						eof=0;
				}
				break;

			case 'p': /* Previous page */
				if (tof)
					redisplay=0;
				else
				{
					eof=0;
					show_header();
					cur_art = old_cur_art - line_no;
					old_cur_art = cur_art;
					if (article[cur_art].rank==0)
						tof=1;
					else
						tof=0;
					for (cur_line=0; cur_line<line_no; cur_line++)
					{
						disp_f(&article[cur_art]);
						cur_art++;
					}
				}
				break;

			case 'N': /* Next group */
			case '>':
				no = 0;
				cur_grp++;
				if (cur_grp>Max_grp)
					cur_grp=0;
				cur_line=0;
				eof=0;
				tof=1;
				show_header();
				cur_art=group[cur_grp].start;
				old_cur_art=cur_art;
				while ((cur_line<line_no) && cur_line<group[cur_grp].art)
				{
					disp_f(&article[cur_art]);
					cur_line++;
					cur_art++;
				}
				if (cur_line!=line_no)
				{
					eof=1;
					for (i=cur_line; i<=line_no; i++)
						printf("\n");
				}
				break;

			case 'P': /* Prev group */
			case '<':
				no = 0;
				if (cur_grp == 0)
					cur_grp=Max_grp;
				else
					cur_grp--;
				cur_line=0;
				eof=0;
				tof=1;
				cur_art=group[cur_grp].start;
				old_cur_art=cur_art;
				show_header();
				while ((cur_line<line_no) && cur_line<group[cur_grp].art)
				{
					disp_f(&article[cur_art]);
					cur_line++;
					cur_art++;
				}
				if (cur_line!=line_no)
				{
					eof=1;
					for (i=cur_line; i<=line_no; i++)
						printf("\n");
				}
				break;

			case 'H':
			case 'h':
				cls();
				/*
				** Help doesnt help much at the moment
				** Well, c'est la vie...
				*/
				printf ("A : redisplay the same page\n"
					      "n : Display next page\n"
								"p : Display previous page \n"
								"N or > : Go to next Newsgroup\n"
								"P or < : Go to previous Newsgroup\n"
								"v : View an article\n"
								"V : Display an article with its full header\n"
								"M : Send a new mail or Reply to an article by mail\n"
								"W : Write an article or a follow-up to an article\n"
								"SPACE : Display the next article\n"
								"s : Save an article to a filename\n"
								"S : Save an article without the header\n"
								"! : Execute a shell command\n"
								"o : See/configure the options\n"
								"Q : Quit \n\n");
				break;

			case 'Q':
			case 'q':
				printf ("\n"); /* To have the prompt on a new line */
				end=1;
				file_list[0]='\0';
				/*
				** Compress the R*.MSG files
				*/
				if (reply_created)
				{
					/*
					** For each newsgroup where there is a reply
					** Add an entry to the REPLIES file
					** The field "reply" is set only for "follow-up" or "posts"
					** Not for mails or replies by mail to the author.
					*/
					if ((fw=fopen("REPLIES", UNIXW))==NULL)
						exit(1);
					for (i=0;i<MAXGRP;i++)
					{
						if(group[i].reply)
						{
							fprintf(fw,"R%s\tnews\tun\t%d\n",
								group[i].prefix,
								group[i].reply);
								/*
								** Add the filename to the list of files
								** to compress
								*/
								sprintf(temp,"R%s.MSG ",group[i].prefix );
								strcat(file_list,temp);
						}
					}
					fclose(fw);
				}
				/*
				** Now look if any mail reply was created too
				*/
				if ((fr=fopen("RMAIL.MSG","r"))!=NULL)
				{
					fclose(fr);
					fw=fopen("REPLIES", UNIXA);
					fprintf(fw,"RMAIL\tmail\tun\t%d\n", mail_reply_no);
					sprintf( temp, "RMAIL.MSG " );
					strcat(file_list,temp);
					fclose(fw);
				}
				if (file_list[0])
				{
					sprintf (temp,"%s %s REPLIES %s",compress_str,Reply,file_list);
					system (temp);
				}

#ifdef ATARI
				closedown_atari();
#endif
				break;

			case 'V':
			case 'v':
				printf ("\n");
				sprintf (value,"%d",no+1);
				fgetstr("No >",temp,value,4,stdin);
				no = atoi(temp);
showit:
				if ((no>0) && (no <= group[cur_grp].art))
				{
					sprintf( temp, "%s.MSG", group[cur_grp].prefix );
					fr=fopen(temp, UNIXR);
					/* Get the starting article number */
					count=group[cur_grp].start+no-1;

					/* save the article into a temp file */
					fseek(fr, article[count].ptr, 0);
					put_into_file("art.tmp",fr,"",
							(choice=='V') ? 0L : NO_HEADER, 
							(struct header *)NULL);
					fclose(fr);

					sprintf (temp,"%s %s",pager_str,"art.tmp");
					system(temp);
					remove("art.tmp");
					/*
					** Show the list again
					*/
					show_header();
					cur_art=old_cur_art;
					for (j=0;j<cur_line;j++)
					{
						disp_f(&article[cur_art]);
						cur_art++;
					}
					if (cur_line!=line_no)
					{
						eof=1;
						for (i=cur_line; i<=line_no; i++)
							printf("\n");
					}
				}
				break;

			case ' ':
				if(no < group[cur_grp].art)
					no++;
				goto showit;

			case 'M': /* Mail somebody */
			case 'm':
				printf ("\n%sN%sew message or %sR%sesponse ?",
					scr(1,3,0),scr(1,6,0),scr(1,3,0),scr(1,6,0));
				fflush(stdout);
				choice=ch_get();
				printf ("\n");
				subjectline[0]='\0';
				if (choice=='r' || choice=='R')
				{
					/* Ask to which mail */
					sprintf (value,"%d",no);
					fgetstr("Reply to No >",temp,value,4,stdin);
					no=atoi(temp);
					if ((no>0) && (no <= group[cur_grp].art) )
					{
						/*
						** Get article and quote it
						*/
						sprintf( temp, "%s.MSG", group[cur_grp].prefix );
						fr=fopen(temp, UNIXR);
						count=group[cur_grp].start+no-1;
						fseek(fr, article[count].ptr, 0);

						/* save the article into a temp file */
						head.subject = line;
						head.from = From;
						head.replyto = Replyto;

						ret = put_into_file( "ed0.tmp",fr, quote_str,
							NO_HEADER | GET_SUBJECT | GET_FROM | GET_REPLYTO, &head);
						fclose(fr);

						if( ret == 0 )
						{
							perror( "Unable to write temp file" );
							break;
						}

						/*
						** Test if the subject line is already a reply
						*/

						remove_cr( line );
						strcpy( temp, line );
						lowerit( temp );

						if (strncmp(temp,"RE:",3)==0)
							sprintf(subjectline,"%s",line);
						else
							sprintf(subjectline,"Re: %s",line);

						if (*Replyto)
							strcpy( To, Replyto);
						else if (*From)
							strcpy( To, From);
						remove_cr( To );
						clean_to( To );

						/*
						** Add the group in the body of the quoted article
						*/
						remove_cr(From);
						fr = fopen( "ed0.tmp","r" );
						fw = fopen( "edit.tmp","w" );
						fprintf( fw, "In %s you wrote:\n", group[cur_grp].name);
						while( fgets(line,255,fr)!=NULL )
						{
							fprintf( fw, "%s", line );
						}
						fclose( fr );
						fclose( fw );
						remove( "ed0.tmp" );
					}
				}
				/* Print the header first */
				fw=fopen ("article.tmp","w");
				if (fw==NULL)
					break;

				if (!*To)
					fgetstr("To >",To,"",60,stdin);
				fgetstr("Subject >",temp,subjectline,60,stdin);
				fprintf (fw,"To: %s\n",To);
				fprintf (fw,"Subject: %s\n",temp);
				fclose(fw);
				To[0]=From[0]=Replyto[0]='\0';

				if (edit_file("edit.tmp","article.tmp"))
					remove("article.tmp");
				else
				{
					/* Add the signature */
					addsig("edit.tmp");

					/* Concatenate article.tmp and edit.tmp */
					fr=fopen("edit.tmp","r");
					fw=fopen("article.tmp","a");
					fprintf (fw,"\n"); /* End of header */
					while (fgets(line,255,fr)!=NULL)
						fprintf(fw,"%s",line);
					fclose(fr);
					fclose(fw);
					remove("edit.tmp");

					/*
					** Add article.tmp to the RMAIL.MSG file
					*/
					sprintf( temp, "RMAIL.MSG" );
					add_file(temp,"article.tmp");
					remove("article.tmp");
					mail_reply_no++;
				}
				/* Then redisplay the list */
				display_page_again(line_no);
				break;

			case 'W': /* write article or follow-up */
			case 'w':
				if( group[cur_grp].enc_type != 'u')
					break;

				printf ("\n%sN%sew article or %sF%sollow-up ?",
					scr(1,3,0),scr(1,6,0),scr(1,3,0),scr(1,6,0));
				choice=ch_get();
				printf ("\n");

				subjectline[0]='\0';
				msgidline[0]='\0';
				strcpy( ngrpline, group[cur_grp].name );

				if (choice=='f' || choice=='F')
				{
					/*
					** Follow-up.
					** Ask to which article
					*/
					sprintf (value,"%d",no);
					fgetstr("Follow up to No >",temp,value,4,stdin);
					no=atoi(temp);
					if ((no>0) && (no <= group[cur_grp].art)
						&& (group[cur_grp].enc_type=='u'))
					{
						/*
						** Get article and quote it
						*/

						ngrpline[0] = followupline[0] = '\0';

						sprintf( temp, "%s.MSG", group[cur_grp].prefix );
						fr=fopen(temp, UNIXR);
						count=group[cur_grp].start+no-1;
						fseek(fr, article[count].ptr, 0);

						/* 
						** save the article into a temp file
						*/
						head.subject = line;
						head.id = msgidline;
						head.from = From;
						head.newsgroups = ngrpline;
						head.followup = followupline;

						ret = put_into_file( "ed0.tmp",fr, quote_str,
							NO_HEADER | GET_SUBJECT | GET_ID | GET_FROM | 
							GET_FOLLOWUP | GET_NEWSGROUP, &head);

						fclose(fr);
						if( ret == 0 )
						{
							perror( "Unable to write temp file" );
							break;
						}

						if( followupline[0] )
							strcpy( ngrpline, followupline );

						/*
						** Test if the subject line is already a reply
						*/

						remove_cr( line );
						strcpy( temp, line );
						lowerit( temp );

						if (strncmp(temp,"re:",3)==0)
							sprintf(subjectline,"%s",line);
						else
							sprintf(subjectline,"Re: %s",line);

						/*
						** Add the author name and group in the body 
						** of the quoted article
						*/
						remove_cr(From);
						fr = fopen( "ed0.tmp","r" );
						fw = fopen( "edit.tmp","w" );
						fprintf( fw, "%s wrote:\n", From);
						while( fgets(line,255,fr)!=NULL )
						{
							fprintf( fw, "%s", line );
						}
						fclose( fr );
						fclose( fw );
						remove( "ed0.tmp" );

					}
				}
				/* Print the header first */
				fw=fopen ("article.tmp","w");
				if (fw==NULL)
					break;

				fflush(stdin);
				fgetstr( "Subject >", temp, subjectline, 60, stdin );
				fprintf( fw, "Newsgroups: %s\n", ngrpline );
				fprintf( fw, "Subject: %s\n", temp );

				/* If its a follow-up, add reference to help threading */
				if (msgidline[0])
					fprintf (fw,"References: %s",msgidline);

				fclose(fw);

				if (edit_file("edit.tmp","article.tmp"))
					remove("article.tmp");
				else
				{
					/* Add the signature */
					addsig("edit.tmp");

					/* Concatenate article.tmp and edit.tmp */
					fr = fopen( "edit.tmp", "r");
					fw = fopen( "article.tmp", "a");
					fprintf( fw,"\n" ); /* End of header */
					while( fgets(line,255,fr)!=NULL )
						fprintf( fw,"%s",line );
					fclose(fr);
					fclose(fw);
					remove("edit.tmp");
					/*
					** Add article.tmp to the reply file
					*/
					sprintf( temp, "R%s.MSG", group[cur_grp].prefix );
					add_file(temp,"article.tmp");
					remove("article.tmp");
					reply_created++;
					group[cur_grp].reply++;
				}
				/* Then redisplay the list */
				display_page_again(line_no);
				break;
			
			case 's': /* Save article */
			case 'S': /* Save with no header */
				printf("\n");

				/* Get article number */
				sprintf (value,"%d",no);
				fgetstr("Save article No >",temp,value,4,stdin);
				no=atoi(temp);

				/* If the number is within range */
				if ((no>0) && (no <= group[cur_grp].art))
				{

					/* Get file name */
					fgetstr("Save article in:",temp,art_save_name,30,stdin);
					if ( temp[0] )
					{
						strcpy(art_save_name, temp);

						sprintf( temp, "%s.MSG", group[cur_grp].prefix );
						fr=fopen(temp, UNIXR);
						count=group[cur_grp].start+no-1;
						fseek(fr, article[count].ptr, 0);

						/*
						** save the article into a temp file
						*/
						if (!put_into_file( art_save_name, fr, "",
							(choice=='S') ? NO_HEADER : 0L,
							(struct header *)NULL))
							perror("Unable to save article");
						fclose(fr);
					}
				}
				/* Then redisplay the list */
				display_page_again(line_no);
				break;

			case '!':
				printf( "\n" );
				fgetstr("Shell cmd:",temp,shell_cmd_name,30,stdin);
				if ( temp[0] )
				{
					strcpy( shell_cmd_name, temp );
					system( temp );
					printf( "Press any key to return to SLNR" );
					fflush( stdout );
					ch_get();
				}
				display_page_again(line_no);
				break;
			
			case 'o': /* Options */
			case 'O':
				option_show_info();
				break;

			default:
				redisplay=0;
				break;

		}
	}
	return(0);
}
